@@ -6,7 +6,7 @@
 # Copyright (C) 1995-1998 The Open Group.   All Rights Reserved.
 # (Please see the file "comm.LICENSE" that accompanied this source,
 #  or http://www.opengroup.org/www/dist_client/caubweb/COPYRIGHT.free.html)
-# Copyright (c) 2003 ActiveState Corporation
+# Copyright (c) 2003-2007 ActiveState Corporation
 #
 # This is the 'comm' package written by Jon Robert LoVerso, placed
 # into its own namespace during integration into tcllib.
@@ -22,9 +22,10 @@
 #
 #	See the manual page comm.n for further details on this package.
 #
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: comm.tcl,v 1.30 2007/08/15 21:33:37 andreas_kupries Exp $
 
-package require Tcl 8.2
+package require Tcl 8.3
+package require snit ; # comm::future objects.
 
 namespace eval ::comm {
     namespace export comm comm_send
@@ -46,6 +47,7 @@ namespace eval ::comm {
 	    acceptVers		{3 2}
 	    defVers		2
 	    defaultEncoding	"utf-8"
+	    defaultSilent   0
 	}
 	set comm(lastport) [expr {[pid] % 32768 + 9999}]
 	# fast check for acceptable versions
@@ -59,12 +61,19 @@ namespace eval ::comm {
     #	lastport		saves last default listening port allocated
     #	debug			enable debug output
     #	chans			list of allocated channels
+    #   future,fid,$fid         List of futures a specific peer is waiting for.
     #
     # Channel instance variables:
     # comm()
     #	$ch,port		listening port (our id)
     #	$ch,socket		listening socket
+    #   $ch,silent      boolean to indicate whether to throw error on
+    #                   protocol negotiation failure
     #	$ch,local		boolean to indicate if port is local
+    #	$ch,interp		interpreter to run received scripts in.
+    #				If not empty we own it! = We destroy it
+    #				with the channel
+    #	$ch,events		List of hoks to run in the 'interp', if defined
     #	$ch,serial		next serial number for commands
     #
     #	$ch,hook,$hook		script for hook $hook
@@ -79,8 +88,8 @@ namespace eval ::comm {
     #	$ch,return,$serial	return codes to go along with result
 
     if {0} {
-	# Propogate result, code, and errorCode.  Can't just eval
-	# otherwise TCL_BREAK gets turrned into TCL_ERROR.
+	# Propagate result, code, and errorCode.  Can't just eval
+	# otherwise TCL_BREAK gets turned into TCL_ERROR.
 	global errorInfo errorCode
 	set code [catch [concat commSend $args] res]
 	return -code $code -errorinfo $errorInfo -errorcode $errorCode $res
@@ -178,6 +187,54 @@ proc ::comm::comm_cmd_debug {chan bool} {
     return [set comm(debug) [string is true -strict $bool]]
 }
 
+# ### ### ### ######### ######### #########
+## API: Setup async result generation for a remotely invoked command.
+
+# (future,fid,<fid>) -> list (future)
+# (current,async)    -> bool (default 0) 
+# (current,state)    -> list (chan fid cmd ser)
+
+proc ::comm::comm_cmd_return_async {chan} {
+    variable comm
+
+    if {![info exists comm(current,async)]} {
+	return -code error "No remote commands processed yet"
+    }
+    if {$comm(current,async)} {
+	# Return the same future which were generated by the first
+	# call.
+	return $comm(current,state)
+    }
+
+    foreach {cmdchan cmdfid cmd ser} $comm(current,state) break
+
+    # Assert that the channel performing the request and the channel
+    # the current command came in are identical. Panic if not.
+
+    if {![string equal $chan $cmdchan]} {
+	return -code error "Internal error: Trying to activate\
+		async return for a command on a different channel"
+    }
+
+    # Establish the future for the command and return a handle for
+    # it. Remember the outstanding futures for a peer, so that we can
+    # cancel them if the peer is lost before the promise implicit in
+    # the future is redeemed.
+
+    set future [::comm::future %AUTO% $chan $cmdfid $cmd $ser]
+
+    lappend comm(future,fid,$cmdfid) $future
+    set     comm(current,state)      $future
+
+    # Mark the current command as using async result return. We do
+    # this last to ensure that all errors in this method are reported
+    # through the regular channels.
+
+    set comm(current,async) 1
+
+    return $future
+}
+
 # hook --
 #
 #	Internal command. Implements 'comm hook'.
@@ -245,15 +302,31 @@ proc ::comm::comm_cmd_destroy {chan} {
     variable comm
     catch {close $comm($chan,socket)}
     comm_cmd_abort $chan
+    if {$comm($chan,interp) != {}} {
+	interp delete $comm($chan,interp)
+    }
     catch {unset comm($chan,port)}
     catch {unset comm($chan,local)}
+    catch {unset comm($chan,silent)}
+    catch {unset comm($chan,interp)}
+    catch {unset comm($chan,events)}
     catch {unset comm($chan,socket)}
+    catch {unset comm($chan,remoteid)}
     unset comm($chan,serial)
+    unset comm($chan,chan)
+    unset comm($chan,encoding)
+    unset comm($chan,listen)
+    # array unset would have been nicer, but is not available in
+    # 8.2/8.3
+    foreach pattern {hook,* interp,* vers,*} {
+	foreach k [array names comm $chan,$pattern] {unset comm($k)}
+    }
     set pos [lsearch -exact $comm(chans) $chan]
     set comm(chans) [lreplace $comm(chans) $pos $pos]
     if {![string equal ::comm::comm $chan]} {
 	rename $chan {}
     }
+    return
 }
 
 # shutdown --
@@ -331,7 +404,10 @@ proc ::comm::comm_cmd_new {chan ch args} {
     set comm($chan,listen) 0
     set comm($chan,socket) ""
     set comm($chan,local)  1
+    set comm($chan,silent)   $comm(defaultSilent)
     set comm($chan,encoding) $comm(defaultEncoding)
+    set comm($chan,interp)   {}
+    set comm($chan,events)   {}
 
     if {[llength $args] > 0} {
 	if {[catch [linsert $args 0 commConfigure $chan 1] err]} {
@@ -390,12 +466,14 @@ proc ::comm::comm_cmd_send {chan args} {
     # This is unneeded - wraps from 2147483647 to -2147483648
     ### if {$comm($chan,serial) == 0x7fffffff} {set comm($chan,serial) 0}
 
-    commDebug {puts stderr "send <[list [list $cmd $ser $args]]>"}
+    commDebug {puts stderr "<$chan> send <[list [list $cmd $ser $args]]>"}
 
     # The double list assures that the command is a single list when read.
     puts  $fid [list [list $cmd $ser $args]]
     flush $fid
 
+    commDebug {puts stderr "<$chan> sent"}
+
     # wait for reply if so requested
 
     if {[string equal command $cmd]} {
@@ -410,7 +488,7 @@ proc ::comm::comm_cmd_send {chan args} {
 	lappend pending $ser
 	set comm($chan,return,$ser) ""		;# we're waiting
 
-	commDebug {puts stderr "--<<waiting $chan $ser>>--"}
+	commDebug {puts stderr "<$chan> --<<waiting $ser>>--"}
 	vwait ::comm::comm($chan,result,$ser)
 
 	# if connection was lost, pending is gone
@@ -420,22 +498,23 @@ proc ::comm::comm_cmd_send {chan args} {
 	}
 
 	commDebug {
-	    puts stderr "result\
+	    puts stderr "<$chan> result\
 		    <$comm($chan,return,$ser);$comm($chan,result,$ser)>"
 	}
-	after idle unset ::comm::comm($chan,result,$ser)
 
 	array set return $comm($chan,return,$ser)
 	unset comm($chan,return,$ser)
+	set thisres $comm($chan,result,$ser)
+	unset comm($chan,result,$ser)
 	switch -- $return(-code) {
-	    "" - 0 {return $comm($chan,result,$ser)}
+	    "" - 0 {return $thisres}
 	    1 {
 		return  -code $return(-code) \
 			-errorinfo $return(-errorinfo) \
 			-errorcode $return(-errorcode) \
-			$comm($chan,result,$ser)
+			$thisres
 	    }
-	    default {return -code $return(-code) $comm($chan,result,$ser)}
+	    default {return -code $return(-code) $thisres}
 	}
     }
 }
@@ -480,14 +559,18 @@ proc ::comm::commConfVars {v t} {
     foreach c [array names comm *,var] {
 	lappend comm(vars) [lindex [split $c ,] 0]
     }
+    return
 }
-::comm::commConfVars port p
-::comm::commConfVars local b
-::comm::commConfVars listen b
-::comm::commConfVars socket ro
-::comm::commConfVars chan ro
-::comm::commConfVars serial ro
+::comm::commConfVars port     p
+::comm::commConfVars local    b
+::comm::commConfVars listen   b
+::comm::commConfVars socket   ro
+::comm::commConfVars chan     ro
+::comm::commConfVars serial   ro
 ::comm::commConfVars encoding enc
+::comm::commConfVars silent   b
+::comm::commConfVars interp   interp
+::comm::commConfVars events   ev
 
 # ::comm::commConfigure --
 #
@@ -527,6 +610,40 @@ proc ::comm::commConfigure {chan {force 0} args} {
 	}
 	set optval [lindex $args $opt]
 	switch $comm($var,var) {
+	    ev {
+		if {![string equal  $optval ""]} {
+		    set err 0
+		    if {[catch {
+			foreach ev $optval {
+			    if {[lsearch -exact {connecting connected incoming eval callback reply lost} $ev] < 0} {
+				set err 1
+				break
+			    }
+			}
+		    }]} {
+			set err 1
+		    }
+		    if {$err} {
+			return -code error \
+				"Non-event to configuration option: -$var"
+		    }
+		}
+		# FRINK: nocheck
+		set $var $optval
+		set skip 1
+	    }
+	    interp {
+		if {
+		    ![string equal  $optval ""] &&
+		    ![interp exists $optval]
+		} {
+		    return -code error \
+			    "Non-interpreter to configuration option: -$var"
+		}
+		# FRINK: nocheck
+		set $var $optval
+		set skip 1
+	    }
 	    b {
 		# FRINK: nocheck
 		set $var [string is true -strict $optval]
@@ -585,6 +702,26 @@ proc ::comm::commConfigure {chan {force 0} args} {
 	}
     }
 
+    foreach var {silent interp events} {
+	# FRINK: nocheck
+	if {[info exists $var] && ([set $var] != $comm($chan,$var))} {
+	    # FRINK: nocheck
+	    set comm($chan,$var) [set ip [set $var]]
+	    if {[string equal $var "interp"] && ($ip != "")} {
+		# Interrogate the interp about its capabilities.
+		#
+		# Like: set, array set, uplevel present ?
+		# Or:   The above, hidden ?
+		#
+		# This is needed to decide how to execute hook scripts
+		# and regular scripts in this interpreter.
+		set comm($chan,interp,set)  [Capability $ip set]
+		set comm($chan,interp,aset) [Capability $ip array]
+		set comm($chan,interp,upl)  [Capability $ip uplevel]
+	    }
+	}
+    }
+
     if {[info exists encoding] &&
 	![string equal $encoding $comm($chan,encoding)]} {
 	# This should not be entered yet
@@ -627,7 +764,7 @@ proc ::comm::commConfigure {chan {force 0} args} {
 	    break
 	}
 	if {[info exists userport] || ![string match "*already in use" $ret]} {
-	    # don't erradicate the class
+	    # don't eradicate the class
 	    if {![string equal ::comm::comm $chan]} {
 		rename $chan {}
 	    }
@@ -643,6 +780,34 @@ proc ::comm::commConfigure {chan {force 0} args} {
     return ""
 }
 
+# ::comm::Capability --
+#
+#	Internal command. Interogate an interp for
+#	the commands needed to execute regular and
+#	hook scripts.
+
+proc ::comm::Capability {interp cmd} {
+    if {[lsearch -exact [interp hidden $interp] $cmd] >= 0} {
+	# The command is present, although hidden.
+	return hidden
+    }
+
+    # The command is not a hidden command. Use info to determine if it
+    # is present as regular command. Note that the 'info' command
+    # itself might be hidden.
+
+    if {[catch {
+	set has [llength [interp eval $interp [list info commands $cmd]]]
+    }] && [catch {
+	set has [llength [interp invokehidden $interp info commands $cmd]]
+    }]} {
+	# Unable to interogate the interpreter in any way. Assume that
+	# the command is not present.
+	set has 0
+    }
+    return [expr {$has ? "ok" : "no"}]
+}
+
 # ::comm::commConnect --
 #
 #	Internal command. Called to connect to a remote interp
@@ -659,12 +824,10 @@ proc ::comm::commConfigure {chan {force 0} args} {
 proc ::comm::commConnect {chan id} {
     variable comm
 
-    commDebug {puts stderr "commConnect $id"}
+    commDebug {puts stderr "<$chan> commConnect $id"}
 
     # process connecting hook now
-    if {[info exists comm($chan,hook,connecting)]} {
-    	eval $comm($chan,hook,connecting)
-    }
+    CommRunHook $chan connecting
 
     if {[info exists comm($chan,peers,$id)]} {
 	return $comm($chan,peers,$id)
@@ -682,13 +845,13 @@ proc ::comm::commConnect {chan id} {
     set fid [socket $host $port]
 
     # process connected hook now
-    if {[info exists comm($chan,hook,connected)]} {
-    	if {[catch $comm($chan,hook,connected) err]} {
-	    global errorInfo
-	    set ei $errorInfo
-	    close $fid
-	    error $err $ei
-	}
+    if {[catch {
+	CommRunHook $chan connected
+    } err]} {
+	global  errorInfo
+	set ei $errorInfo
+	close $fid
+	error $err $ei
     }
 
     # commit new connection
@@ -718,24 +881,26 @@ proc ::comm::commConnect {chan id} {
 proc ::comm::commIncoming {chan fid addr remport} {
     variable comm
 
-    commDebug {puts stderr "commIncoming $chan $fid $addr $remport"}
+    commDebug {puts stderr "<$chan> commIncoming $fid $addr $remport"}
 
     # process incoming hook now
-    if {[info exists comm($chan,hook,incoming)]} {
-    	if {[catch $comm($chan,hook,incoming) err]} {
-	    global errorInfo
-	    set ei $errorInfo
-	    close $fid
-	    error $err $ei
-	}
+    if {[catch {
+	CommRunHook $chan incoming
+    } err]} {
+	global errorInfo
+	set ei $errorInfo
+	close $fid
+	error $err $ei
     }
 
     # a list of offered proto versions is the first word of first line
     # remote id is the second word of first line
     # rest of first line is ignored
-    set protoline [gets $fid]
+    set protoline   [gets $fid]
     set offeredvers [lindex $protoline 0]
-    set remid [lindex $protoline 1]
+    set remid       [lindex $protoline 1]
+
+    commDebug {puts stderr "<$chan> offered <$protoline>"}
 
     # use the first supported version in the offered list
     foreach v $offeredvers {
@@ -746,6 +911,8 @@ proc ::comm::commIncoming {chan fid addr remport} {
     }
     if {![info exists vers]} {
 	close $fid
+	if {[info exists comm($chan,silent)] && 
+	    [string is true -strict $comm($chan,silent)]} then return
 	error "Unknown offered protocols \"$protoline\" from $addr/$remport"
     }
 
@@ -766,8 +933,8 @@ proc ::comm::commIncoming {chan fid addr remport} {
 	puts stderr "peers=$comm($chan,peers,$id) port=$comm($chan,port)"
 
 	# To avoid the race, we really want to terminate one connection.
-	# However, both sides are commited to using it.
-	# commConnect needs to be sychronous and detect the close.
+	# However, both sides are committed to using it.
+	# commConnect needs to be synchronous and detect the close.
 	# close $fid
 	# return $comm($chan,peers,$id)
     }
@@ -804,7 +971,7 @@ proc ::comm::commIncoming {chan fid addr remport} {
 proc ::comm::commNewConn {chan id fid} {
     variable comm
 
-    commDebug {puts stderr "commNewConn $id $fid"}
+    commDebug {puts stderr "<$chan> commNewConn $id $fid"}
 
     # There can be a race condition two where comms connect to each other
     # simultaneously.  This code favors our outgoing connection.
@@ -838,12 +1005,16 @@ proc ::comm::commNewConn {chan id fid} {
 proc ::comm::commLostConn {chan fid reason} {
     variable comm
 
-    commDebug {puts stderr "commLostConn $fid $reason"}
+    commDebug {puts stderr "<$chan> commLostConn $fid $reason"}
 
     catch {close $fid}
 
     set id $comm($chan,fids,$fid)
 
+    # Invoke the callbacks of all commands which have such and are
+    # still waiting for a response from the lost peer. Use an
+    # appropriate error.
+
     foreach s $comm($chan,pending,$id) {
 	if {[string equal "callback" [lindex $s end]]} {
 	    set ser [lindex $s 0]
@@ -856,7 +1027,7 @@ proc ::comm::commLostConn {chan fid reason} {
 			      -errorinfo "" \
 			      -result    $reason \
 			     ]
-		if {[catch {uplevel #0 $comm($chan,return,$ser) $args} err]} {
+		if {[catch {uplevel \#0 $comm($chan,return,$ser) $args} err]} {
 		    commBgerror $err
 		}
 	    }
@@ -870,8 +1041,15 @@ proc ::comm::commLostConn {chan fid reason} {
     catch {unset comm($chan,peers,$id)}		;# race condition
     catch {unset comm($chan,buf,$fid)}
 
+    # Cancel all outstanding futures for requests which were made by
+    # the lost peer, if there are any. This does not destroy
+    # them. They will stay around until the long-running operations
+    # they belong too kill them.
+
+    CancelFutures $fid
+
     # process lost hook now
-    catch {catch $comm($chan,hook,lost)}
+    catch {CommRunHook $chan lost}
 
     return $reason
 }
@@ -892,6 +1070,26 @@ proc ::comm::commBgerror {err} {
     }
 }
 
+# CancelFutures: Mark futures associated with a comm channel as
+# expired, done when the connection to the peer has been lost. The
+# marked futures will not generate result anymore. They will also stay
+# around until destroyed by the script they belong to.
+
+proc ::comm::CancelFutures {fid} {
+    variable comm
+    if {![info exists comm(future,fid,$fid)]} return
+
+    commDebug {puts stderr "\tCanceling futures: [join $comm(future,fid,$fid) \
+                         "\n\t                 : "]"}
+
+    foreach future $comm(future,fid,$fid) {
+	$future Cancel
+    }
+
+    unset comm(future,fid,$fid)
+    return
+}
+
 ###############################################################################
 
 # ::comm::commCollect --
@@ -913,13 +1111,17 @@ proc ::comm::commCollect {chan fid} {
 
     # Tcl8 may return an error on read after a close
     if {[catch {read $fid} nbuf] || [eof $fid]} {
+	commDebug {puts stderr "<$chan> collect/lost eof $fid = [eof $fid]"}
+	commDebug {puts stderr "<$chan> collect/lost nbuf = <$nbuf>"}
+	commDebug {puts stderr "<$chan> collect/lost [fconfigure $fid]"}
+
 	fileevent $fid readable {}		;# be safe
 	commLostConn $chan $fid "target application died or connection lost"
 	return
     }
     append data $nbuf
 
-    commDebug {puts stderr "collect <$data>"}
+    commDebug {puts stderr "<$chan> collect <$data>"}
 
     # If data contains at least one complete command, we will
     # be able to take off the first element, which is a list holding
@@ -933,7 +1135,7 @@ proc ::comm::commCollect {chan fid} {
     # it could potentially cause a deadlock.
 
     while {![catch {set cmd [lindex $data 0]}]} {
-	commDebug {puts stderr "cmd <$data>"}
+	commDebug {puts stderr "<$chan> cmd <$data>"}
 	if {[string equal "" $cmd]} break
 	if {[info complete $cmd]} {
 	    set data [lreplace $data 0 0]
@@ -962,7 +1164,7 @@ proc ::comm::commExec {chan fid remoteid buf} {
     variable comm
 
     # buffer should contain:
-    #	send # {cmd}		execute cmd and send reply with serial #
+    #	send  # {cmd}		execute cmd and send reply with serial #
     #	async # {cmd}		execute cmd but send no reply
     #	reply # {cmd}		execute cmd as reply to serial #
 
@@ -973,18 +1175,21 @@ proc ::comm::commExec {chan fid remoteid buf} {
     set buffer [lindex $buf 0]
 
     # Save remoteid for "comm remoteid".  This will only be valid
-    # if retrieved before any additional events occur # on this channel.
+    # if retrieved before any additional events occur on this channel.
     # N.B. we could have already lost the connection to remote, making
     # this id be purely informational!
     set comm($chan,remoteid) [set id $remoteid]
 
-    commDebug {puts stderr "exec <$cmd,$ser,$buf>"}
+    # Save state for possible async result generation
+    AsyncPrepare $chan $fid $cmd $ser
+
+    commDebug {puts stderr "<$chan> exec <$cmd,$ser,$buf>"}
 
     switch -- $cmd {
 	send - async - command {}
 	callback {
 	    if {![info exists comm($chan,return,$ser)]} {
-	        commDebug {puts stderr "No one waiting for serial \"$ser\""}
+	        commDebug {puts stderr "<$chan> No one waiting for serial \"$ser\""}
 		return
 	    }
 
@@ -1000,12 +1205,10 @@ proc ::comm::commExec {chan fid remoteid buf} {
 		set return($sw) $val
 	    }
 
-	    if {[info exists comm($chan,hook,callback)]} {
-		catch $comm($chan,hook,callback)
-	    }
+	    catch {CommRunHook $chan callback}
 
 	    # this wakes up the sender
-	    commDebug {puts stderr "--<<wakeup $chan $ser>>--"}
+	    commDebug {puts stderr "<$chan> --<<wakeup $ser>>--"}
 
 	    # the return holds the callback command
 	    # string map the optional %-subs
@@ -1017,7 +1220,7 @@ proc ::comm::commExec {chan fid remoteid buf} {
 			  -errorinfo $return(-errorinfo) \
 			  -result    $ret \
 			 ]
-	    set code [catch {uplevel #0 $comm($chan,return,$ser) $args} err]
+	    set code [catch {uplevel \#0 $comm($chan,return,$ser) $args} err]
 	    catch {unset comm($chan,return,$ser)}
 
 	    # remove pending serial
@@ -1035,7 +1238,7 @@ proc ::comm::commExec {chan fid remoteid buf} {
 	}
 	reply {
 	    if {![info exists comm($chan,return,$ser)]} {
-	        commDebug {puts stderr "No one waiting for serial \"$ser\""}
+	        commDebug {puts stderr "<$chan> No one waiting for serial \"$ser\""}
 		return
 	    }
 
@@ -1051,12 +1254,10 @@ proc ::comm::commExec {chan fid remoteid buf} {
 		set return($sw) $val
 	    }
 
-	    if {[info exists comm($chan,hook,reply)]} {
-		catch $comm($chan,hook,reply)
-	    }
+	    catch {CommRunHook $chan reply}
 
 	    # this wakes up the sender
-	    commDebug {puts stderr "--<<wakeup $chan $ser>>--"}
+	    commDebug {puts stderr "<$chan> --<<wakeup $ser>>--"}
 	    set comm($chan,result,$ser) $ret
 	    set comm($chan,return,$ser) [array get return]
 	    return
@@ -1066,36 +1267,91 @@ proc ::comm::commExec {chan fid remoteid buf} {
 	    return
 	}
 	default {
-	    commDebug {puts stderr "unknown command; discard \"$cmd\""}
+	    commDebug {puts stderr "<$chan> unknown command; discard \"$cmd\""}
 	    return
 	}
     }
 
     # process eval hook now
+    set done 0
+    set err  0
     if {[info exists comm($chan,hook,eval)]} {
-    	set err [catch $comm($chan,hook,eval) ret]
-	commDebug {puts stderr "eval hook res <$err,$ret>"}
+	set err [catch {CommRunHook $chan eval} ret]
+	commDebug {puts stderr "<$chan> eval hook res <$err,$ret>"}
 	switch $err {
-	    1 {				;# error
+	    1 {
+		# error
 		set done 1
 	    }
-	    2 - 3 {			;# return / break
+	    2 - 3 {
+		# return / break
 		set err 0
 		set done 1
 	    }
 	}
     }
 
+    commDebug {puts stderr "<$chan> hook(eval) done=$done, err=$err"}
+
     # exec command
-    if {![info exists done]} {
+    if {!$done} {
+	commDebug {puts stderr "<$chan> exec ($buffer)"}
+
 	# Sadly, the uplevel needs to be in the catch to access the local
 	# variables buffer and ret.  These cannot simply be global because
 	# commExec is reentrant (i.e., they could be linked to an allocated
 	# serial number).
-	set err [catch [concat [list uplevel #0] $buffer] ret]
+
+	if {$comm($chan,interp) == {}} {
+	    # Main interpreter
+	    set thecmd [concat [list uplevel \#0] $buffer]
+	    set err    [catch $thecmd ret]
+	} else {
+	    # Redirect execution into the configured slave
+	    # interpreter. The exact command used depends on the
+	    # capabilities of the interpreter. A best effort is made
+	    # to execute the script in the global namespace.
+	    set interp $comm($chan,interp)
+
+	    if {$comm($chan,interp,upl) == "ok"} {
+		set thecmd [concat [list uplevel \#0] $buffer]
+		set err [catch {interp eval $interp $thecmd} ret]
+	    } elseif {$comm($chan,interp,aset) == "hidden"} {
+		set thecmd [linsert $buffer 0 interp invokehidden $interp uplevel \#0]
+		set err [catch $thecmd ret]
+	    } else {
+		set thecmd [concat [list interp eval $interp] $buffer]
+		set err [catch $thecmd ret]
+	    }
+	}
     }
 
-    commDebug {puts stderr "res <$err,$ret>"}
+    # Check and handle possible async result generation.
+    if {[AsyncCheck]} return
+
+    commSendReply $chan $fid $cmd $ser $err $ret
+    return
+}
+
+# ::comm::commSendReply --
+#
+#	Internal command. Executed to construct and send the reply
+#	for a command.
+#
+# Arguments:
+#	fid		channel handle of the socket we are replying to.
+#	cmd		The type of request (send, command) we are replying to.
+#	ser		Serial number of the request the reply is for.
+#	err		result code to place into the reply.
+#	ret		result value to place into the reply.
+#
+# Results:
+#	None.
+
+proc ::comm::commSendReply {chan fid cmd ser err ret} {
+    variable comm
+
+    commDebug {puts stderr "<$chan> res <$err,$ret> /$cmd"}
 
     # The double list assures that the command is a single list when read.
     if {[string equal send $cmd] || [string equal command $cmd]} {
@@ -1114,17 +1370,279 @@ proc ::comm::commExec {chan fid remoteid buf} {
 	    } else {
 		set reply callback
 	    }
-	    puts $fid [list [list $reply $ser $return]]
+	    puts  $fid [list [list $reply $ser $return]]
 	    flush $fid
 	}
+	commDebug {puts stderr "<$chan> reply sent"}
     }
 
     if {$err == 1} {
 	commBgerror $ret
     }
+    commDebug {puts stderr "<$chan> exec complete"}
     return
 }
 
+proc ::comm::CommRunHook {chan event} {
+    variable comm
+
+    # The documentation promises the hook scripts to have access to a
+    # number of internal variables. For a regular hook we simply
+    # execute it in the calling level to fulfill this. When the hook
+    # is redirected into an interpreter however we do a best-effort
+    # copying of the variable values into the interpreter. Best-effort
+    # because the 'set' command may not be available in the
+    # interpreter, not even hidden.
+
+    if {![info exists comm($chan,hook,$event)]} return
+    set cmd    $comm($chan,hook,$event)
+    set interp $comm($chan,interp)
+    commDebug {puts stderr "<$chan> hook($event) run <$cmd>"}
+
+    if {
+	($interp != {}) &&
+	([lsearch -exact $comm($chan,events) $event] >= 0)
+    } {
+	# Best-effort to copy the context into the interpreter for
+	# access by the hook script.
+	set vars   {
+	    addr buffer chan cmd fid host
+	    id port reason remport ret var
+	}
+
+	if {$comm($chan,interp,set) == "ok"} {
+	    foreach v $vars {
+		upvar 1 $v V
+		if {![info exists V]} continue
+		interp eval $interp [list set $v $V]
+	    }
+	} elseif {$comm($chan,interp,set) == "hidden"} {
+	    foreach v $vars {
+		upvar 1 $v V
+		if {![info exists V]} continue
+		interp invokehidden $interp set $v $V
+	    }
+	}
+	upvar 1 return AV
+	if {[info exists AV]} {
+	    if {$comm($chan,interp,aset) == "ok"} {
+		interp eval $interp [list array set return [array get AV]]
+	    } elseif {$comm($chan,interp,aset) == "hidden"} {
+		interp invokehidden $interp array set return [array get AV]
+	    }
+	}
+
+	commDebug {puts stderr "<$chan> /interp $interp"}
+	set code [catch {interp eval $interp $cmd} res]
+    } else {
+	commDebug {puts stderr "<$chan> /main"}
+	set code [catch {uplevel 1 $cmd} res]
+    }
+
+    # Perform the return code propagation promised
+    # to the hook scripts.
+    switch -exact -- $code {
+	0 {}
+	1 {
+	    return -errorinfo $::errorInfo -errorcode $::errorCode -code error $res
+	}
+	3 {return}
+	4 {}
+	default {return -code $code $res}
+    }
+    return
+}
+
+# ### ### ### ######### ######### #########
+## Hooks to link async return and future processing into the regular
+## system.
+
+# AsyncPrepare, AsyncCheck: Initialize state information for async
+# return upon start of a remote invokation, and checking the state for
+# async return.
+
+proc ::comm::AsyncPrepare {chan fid cmd ser} {
+    variable comm
+    set comm(current,async) 0
+    set comm(current,state) [list $chan $fid $cmd $ser]
+    return
+}
+
+proc ::comm::AsyncCheck {} {
+    # Check if the executed command notified us of an async return. If
+    # not we let the regular return processing handle the end of the
+    # script. Otherwise we stop the caller from proceeding, preventing
+    # a regular return.
+
+    variable comm
+    if {!$comm(current,async)} {return 0}
+    return 1
+}
+
+# FutureDone: Action taken by an uncanceled future to deliver the
+# generated result to the proper invoker. This also removes the future
+# from the list of pending futures for the comm channel.
+
+proc comm::FutureDone {future chan fid cmd sid rcode rvalue} {
+    variable comm
+    commSendReply $chan $fid $cmd $sid $rcode $rvalue
+
+    set pos [lsearch -exact $comm(future,fid,$fid) $future]
+    set comm(future,fid,$fid) [lreplace $comm(future,fid,$fid) $pos $pos]
+    return
+}
+
+# ### ### ### ######### ######### #########
+## Hooks to save command state across nested eventloops a remotely
+## invoked command may run before finally activating async result
+## generation.
+
+# DANGER !! We have to refer to comm internals using fully-qualified
+# names because the wrappers will execute in the global namespace
+# after their installation.
+
+proc ::comm::Vwait {varname} {
+    variable ::comm::comm
+
+    set hasstate [info exists comm(current,async)]
+    set hasremote 0
+    if {$hasstate} {
+	set chan     [lindex $comm(current,state) 0]
+	set async    $comm(current,async)
+	set state    $comm(current,state)
+	set hasremote [info exists comm($chan,remoteid)]
+	if {$hasremote} {
+	    set remoteid $comm($chan,remoteid)
+	}
+    }
+
+    set code [catch {uplevel 1 [list ::comm::VwaitOrig $varname]} res]
+
+    if {$hasstate} {
+	set comm(current,async)  $async
+	set comm(current,state)	 $state
+    }
+    if {$hasremote} {
+	set comm($chan,remoteid) $remoteid
+    }
+
+    return -code $code $res
+}
+
+proc ::comm::Update {args} {
+    variable ::comm::comm
+
+    set hasstate [info exists comm(current,async)]
+    set hasremote 0
+    if {$hasstate} {
+	set chan     [lindex $comm(current,state) 0]
+	set async    $comm(current,async)
+	set state    $comm(current,state)
+
+	set hasremote [info exists comm($chan,remoteid)]
+	if {$hasremote} {
+	    set remoteid $comm($chan,remoteid)
+	}
+    }
+
+    set code [catch {uplevel 1 [linsert $args 0 ::comm::UpdateOrig]} res]
+
+    if {$hasstate} {
+	set comm(current,async)  $async
+	set comm(current,state)	 $state
+    }
+    if {$hasremote} {
+	set comm($chan,remoteid) $remoteid
+    }
+
+    return -code $code $res
+}
+
+# Install the wrappers.
+
+proc ::comm::InitWrappers {} {
+    rename ::vwait       ::comm::VwaitOrig
+    rename ::comm::Vwait ::vwait
+
+    rename ::update       ::comm::UpdateOrig
+    rename ::comm::Update ::update
+
+    proc ::comm::InitWrappers {} {}
+    return
+}
+
+# ### ### ### ######### ######### #########
+## API: Future objects.
+
+snit::type comm::future {
+    option -command -default {}
+
+    constructor {chan fid cmd ser} {
+	set xfid  $fid
+	set xcmd  $cmd
+	set xser  $ser
+	set xchan $chan
+	return
+    }
+
+    destructor {
+	if {!$canceled} {
+	    return -code error \
+		    "Illegal attempt to destroy unresolved future \"$self\""
+	}
+    }
+
+    method return {args} {
+	# Syntax:             | 0
+	#       : -code x     | 2
+	#       : -code x val | 3
+	#       :         val | 4
+	# Allowing multiple -code settings, last one is taken.
+
+	set rcode  0
+	set rvalue {}
+
+	while {[lindex $args 0] == "-code"} {
+	    set rcode [lindex $args 1]
+	    set args  [lrange $args 2 end]
+	}
+	if {[llength $args] > 1} {
+	    return -code error "wrong\#args, expected \"?-code errcode? ?result?\""
+	}
+	if {[llength $args] == 1} {
+	    set rvalue [lindex $args 0]
+	}
+
+	if {!$canceled} {
+	    comm::FutureDone $self $xchan $xfid $xcmd $xser $rcode $rvalue
+	    set canceled 1
+	}
+	# assert: canceled == 1
+	$self destroy
+	return
+    }
+
+    variable xfid  {}
+    variable xcmd  {}
+    variable xser  {}
+    variable xchan {}
+    variable canceled 0
+
+    # Internal method for use by comm channels. Marks the future as
+    # expired, no peer to return a result back to.
+
+    method Cancel {} {
+	set canceled 1
+	if {![llength $options(-command)]} {return}
+	uplevel #0 [linsert $options(-command) end $self]
+	return
+    }
+}
+
+# ### ### ### ######### ######### #########
+## Setup
+::comm::InitWrappers
+
 ###############################################################################
 #
 # Finish creating "comm" using the default port for this interp.
@@ -1142,4 +1660,4 @@ if {![info exists ::comm::comm(comm,port)]} {
 }
 
 #eof
-package provide comm 4.2
+package provide comm 4.5.6
