@@ -29,7 +29,7 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id: dns.tcl,v 1.30 2005/09/30 05:36:38 andreas_kupries Exp $
+# $Id: dns.tcl,v 1.35 2007/08/26 00:44:34 patthoyts Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 package require logger;                 # tcllib 1.3
@@ -38,8 +38,8 @@ package require uri::urn;               # tcllib 1.2
 package require ip;                     # tcllib 1.7
 
 namespace eval ::dns {
-    variable version 1.3.0
-    variable rcsid {$Id: dns.tcl,v 1.30 2005/09/30 05:36:38 andreas_kupries Exp $}
+    variable version 1.3.2
+    variable rcsid {$Id: dns.tcl,v 1.35 2007/08/26 00:44:34 patthoyts Exp $}
 
     namespace export configure resolve name address cname \
         status reset wait cleanup errorcode
@@ -73,7 +73,7 @@ namespace eval ::dns {
         A 1  NS 2  MD 3  MF 4  CNAME 5  SOA 6  MB 7  MG 8  MR 9 
         NULL 10  WKS 11  PTR 12  HINFO 13  MINFO 14  MX 15  TXT 16
         SPF 16 AAAA 28 SRV 33 IXFR 251 AXFR 252  MAILB 253  MAILA 254
-        * 255
+        ANY 255 * 255
     } 
 
     variable classes
@@ -376,9 +376,7 @@ proc ::dns::result {token args} {
 #  Get the status of the request.
 #
 proc ::dns::status {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     return $state(status)
 }
 
@@ -386,9 +384,7 @@ proc ::dns::status {token} {
 #  Get the error message. Empty if no error.
 #
 proc ::dns::error {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     if {[info exists state(error)]} {
 	return $state(error)
     }
@@ -399,9 +395,7 @@ proc ::dns::error {token} {
 #  Get the error code. This is 0 for a successful transaction.
 #
 proc ::dns::errorcode {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     set flags [Flags $token]
     set ndx [lsearch -exact $flags errorcode]
     incr ndx
@@ -412,9 +406,7 @@ proc ::dns::errorcode {token} {
 #  Reset a connection with optional reason.
 #
 proc ::dns::reset {token {why reset} {errormsg {}}} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     set state(status) $why
     if {[string length $errormsg] > 0 && ![info exists state(error)]} {
         set state(error) $errormsg
@@ -427,9 +419,7 @@ proc ::dns::reset {token {why reset} {errormsg {}}} {
 #  Wait for a request to complete and return the status.
 #
 proc ::dns::wait {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
 
     if {$state(status) == "connect"} {
         vwait [subst $token](status)
@@ -442,10 +432,10 @@ proc ::dns::wait {token} {
 #  Remove any state associated with this token.
 #
 proc ::dns::cleanup {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
     if {[info exists state]} {
+        catch {close $state(sock)}
+        catch {after cancel $state(after)}
         unset state
     }
 }
@@ -670,10 +660,6 @@ proc ::dns::TcpTransmit {token} {
     variable $token
     upvar 0 $token state
 
-    # For TCP the message must be prefixed with a 16bit length field.
-    set req [binary format S [string length $state(request)]]
-    append req $state(request)
-
     # setup the timeout
     if {$state(-timeout) > 0} {
         set state(after) [after $state(-timeout) \
@@ -682,16 +668,36 @@ proc ::dns::TcpTransmit {token} {
                                    "operation timed out"]]
     }
 
-    set s [socket $state(-nameserver) $state(-port)]
-    fconfigure $s -blocking 0 -translation binary -buffering none
+    # Sometimes DNS servers drop TCP requests. So it's better to
+    # use asynchronous connect
+    set s [socket -async $state(-nameserver) $state(-port)]
+    fileevent $s writable [list [namespace origin TcpConnected] $token $s]
     set state(sock) $s
     set state(status) connect
 
+    return $token
+}
+
+proc ::dns::TcpConnected {token s} {
+    variable $token
+    upvar 0 $token state
+
+    fileevent $s writable {}
+    if {[catch {fconfigure $s -peername}]} {
+	# TCP connection failed
+        Finish $token "can't connect to server"
+	return
+    }
+
+    fconfigure $s -blocking 0 -translation binary -buffering none
+
+    # For TCP the message must be prefixed with a 16bit length field.
+    set req [binary format S [string length $state(request)]]
+    append req $state(request)
+
     puts -nonewline $s $req
 
     fileevent $s readable [list [namespace current]::TcpEvent $token]
-    
-    return $token
 }
 
 # -------------------------------------------------------------------------
@@ -840,7 +846,12 @@ proc ::dns::TcpEvent {token} {
             if {[string length $state(reply)] >= $state(size)} {
                 binary scan $result S id
                 set id [expr {$id & 0xFFFF}]
-                Receive [namespace current]::$id
+                if {$id != [expr {$state(id) & 0xFFFF}]} {
+                    ${log}::error "received packed with incorrect id"
+                }
+                # bug #1158037 - doing this causes problems > 65535 requests!
+                #Receive [namespace current]::$id
+                Receive $token
             } else {
                 ${log}::debug "Incomplete tcp read:\
                    [string length $state(reply)] should be $state(size)"
@@ -873,7 +884,12 @@ proc ::dns::UdpEvent {token} {
 
     binary scan $payload S id
     set id [expr {$id & 0xFFFF}]
-    Receive [namespace current]::$id
+    if {$id != [expr {$state(id) & 0xFFFF}]} {
+        ${log}::error "received packed with incorrect id"
+    }
+    # bug #1158037 - doing this causes problems > 65535 requests!
+    #Receive [namespace current]::$id
+    Receive $token
 }
     
 # -------------------------------------------------------------------------
@@ -1106,6 +1122,9 @@ proc ::dns::ReadAnswer {nitems data indexvar {raw 0}} {
                     lappend rdata target [ReadName data $x off]
                     incr x $off
                 }
+                TXT {
+                    set rdata [ReadString data $index $rdlength]
+                }
                 SOA {
                     set x $index
                     set rdata [list MNAME [ReadName data $x off]]
@@ -1211,36 +1230,62 @@ proc ::dns::ReadName {datavar index usedvar} {
     return [join $r .]
 }
 
+proc ::dns::ReadString {datavar index length} {
+    upvar $datavar data
+    set startindex $index
+
+    set r {}
+    set max [expr {$index + $length}]
+
+    while {$index < $max} {
+        binary scan [string range $data $index end] c len
+        set len [expr {$len & 0xFF}]
+        incr index
+
+        if {$len != 0} {
+            append r [string range $data $index [expr {$index + $len - 1}]]
+            incr index $len
+        }
+    }
+    return $r
+}
+
 # -------------------------------------------------------------------------
 
-# Experimental support for finding the nameservers
+# Support for finding the local nameservers
 #
 # For unix we can just parse the /etc/resolv.conf if it exists.
-# Of couse, some unices use /etc/resolver and other things (NIS for instance)
+# Of course, some unices use /etc/resolver and other things (NIS for instance)
 # On Windows, we can examine the Internet Explorer settings from the registry.
 #
 switch -exact $::tcl_platform(platform) {
     windows {
         proc ::dns::nameservers {} {
             package require registry
-            set base {HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services\Tcpip}
-            set param "$base\\Parameters"
+            set base {HKEY_LOCAL_MACHINE\System\CurrentControlSet\Services}
+            set param "$base\\Tcpip\\Parameters"
             set interfaces "$param\\Interfaces"
             set nameservers {}
-            AppendRegistryValue $param NameServer nameservers
-            AppendRegistryValue $param DhcpNameServer nameservers
-            foreach i [registry keys $interfaces] {
-                AppendRegistryValue "$interfaces\\$i" NameServer nameservers
-                AppendRegistryValue "$interfaces\\$i" DhcpNameServer nameservers
+            if {[string equal $::tcl_platform(os) "Windows NT"]} {
+                AppendRegistryValue $param NameServer nameservers
+                AppendRegistryValue $param DhcpNameServer nameservers
+                foreach i [registry keys $interfaces] {
+                    AppendRegistryValue "$interfaces\\$i" NameServer nameservers
+                    AppendRegistryValue "$interfaces\\$i" DhcpNameServer nameservers
+                }
+            } else {
+                set param "$base\\VxD\\MSTCP"
+                AppendRegistryValue $param NameServer nameservers
             }
-            
             return $nameservers
         }
         proc ::dns::AppendRegistryValue {key val listName} {
             upvar $listName lst
             if {![catch {registry get $key $val} v]} {
-                if {[lsearch -exact $lst $v] == -1} {
-                    set lst [concat $lst $v]
+                foreach ns [split $v ", "] {
+                    if {[lsearch -exact $lst $ns] == -1} {
+                        lappend lst $ns
+                    }
                 }
             }
         }
@@ -1258,6 +1303,9 @@ switch -exact $::tcl_platform(platform) {
                 }
                 close $f
             }
+            if {[llength $nameservers] < 1} {
+                lappend nameservers 127.0.0.1
+            }
             return $nameservers
         }
     }
@@ -1364,6 +1412,8 @@ proc ::uri::JoinDns {args} {
 
 # -------------------------------------------------------------------------
 
+catch {dns::configure -nameserver [lindex [dns::nameservers] 0]}
+
 package provide dns $dns::version
 
 # -------------------------------------------------------------------------
