@@ -9,15 +9,15 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id: ip.tcl,v 1.10 2006/09/19 23:36:15 andreas_kupries Exp $
+# $Id: ip.tcl,v 1.12 2007/08/20 20:03:04 andreas_kupries Exp $
 
 # @mdgen EXCLUDE: ipMoreC.tcl
 
 package require Tcl 8.2;                # tcl minimum version
 
 namespace eval ip {
-    variable version 1.1.1
-    variable rcsid {$Id: ip.tcl,v 1.10 2006/09/19 23:36:15 andreas_kupries Exp $}
+    variable version 1.1.2
+    variable rcsid {$Id: ip.tcl,v 1.12 2007/08/20 20:03:04 andreas_kupries Exp $}
 
     namespace export is version normalize equal type contract mask
     #catch {namespace ensemble create}
@@ -271,7 +271,9 @@ proc ::ip::Normalize {ip {version 0}} {
 
 proc ::ip::Normalize4 {ip} {
     set octets [split $ip .]
-    if {[llength $octets] != 4} {
+    if {[llength $octets] > 4} {
+        return -code error "invalid ip address \"$ip\""
+    } elseif {[llength $octets] < 4} {
         set octets [lrange [concat $octets 0 0 0] 0 3]
     }
     foreach oct $octets {
