@@ -2,16 +2,22 @@
 #
 #	Implementation of docidx objects for Tcl.
 #
-# Copyright (c) 2003 Andreas Kupries <andreas_kupries@sourceforge.net>
+# Copyright (c) 2003-2007 Andreas Kupries <andreas_kupries@sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: docidx.tcl,v 1.14 2007/08/20 20:11:59 andreas_kupries Exp $
 
 package require Tcl 8.2
 package require textutil::expander
 
+# @mdgen OWNER: api_idx.tcl
+# @mdgen OWNER: checker_idx.tcl
+# @mdgen OWNER: mpformats/*.tcl
+# @mdgen OWNER: mpformats/*.msg
+# @mdgen OWNER: mpformats/idx.*
+
 namespace eval ::doctools {}
 namespace eval ::doctools::idx {
     # Data storage in the doctools::idx module
@@ -385,9 +391,9 @@ proc ::doctools::idx::_format {name text} {
 	    # Filter for checker errors and reduce them to the essential message.
 
 	    if {![regexp {^Error in} $msg]}          {return -code error $msg}
-	    set msg [join [lrange [split $msg \n] 2 end]]
+	    #set msg [join [lrange [split $msg \n] 2 end]]
 
-	    if {![regexp {^--> \(FmtError\) } $msg]} {return -code error @$msg}
+	    if {![regexp {^--> \(FmtError\) } $msg]} {return -code error "Docidx $msg"}
 	    set msg [lindex [split $msg \n] 0]
 	    regsub {^--> \(FmtError\) } $msg {} msg
 
@@ -533,7 +539,7 @@ proc ::doctools::idx::LookupFormat {name format} {
 #	formatting engine
 #
 # Arguments:
-#	name	Name of the docidx object to manipulaye
+#	name	Name of the docidx object to manipulate
 #	format	Name of file containing the code of the engine
 #
 # Results:
@@ -636,7 +642,7 @@ proc ::doctools::idx::SetupFormatter {name format} {
 #	docidx formatting commands
 #
 # Arguments:
-#	name	Name of the docidx object to manipulaye
+#	name	Name of the docidx object to manipulate
 #
 # Results:
 #	None.
@@ -691,7 +697,7 @@ proc ::doctools::idx::SetupChecker {name} {
 #	Create and initializes the expander for input
 #
 # Arguments:
-#	name	Name of the docidx object to manipulaye
+#	name	Name of the docidx object to manipulate
 #
 # Results:
 #	None.
@@ -730,7 +736,7 @@ proc ::doctools::idx::SearchPaths {name} {
 
 # ::doctools::idx::FmtError --
 #
-#	API for checker. Called when an error occured.
+#	API for checker. Called when an error occurred.
 #
 # Arguments:
 #	name	Name of the docidx object to query.
@@ -896,4 +902,4 @@ namespace eval ::doctools::idx {
     catch {search [file join $here                             mpformats]}
 }
 
-package provide doctools::idx 0.1
+package provide doctools::idx 0.3
