@@ -7,15 +7,12 @@
 # ### ### ### ######### ######### #########
 ## Requisites
 
-package require grammar::fa  ; # Tcllib | FA containers
-package require struct::list ; # Tcllib | Extended list operations.
-package require struct::set  ; # Tcllib | Extended set operations.
+package require struct::list ; # Extended list operations.
+package require struct::set  ; # Extended set operations.
 
 # ### ### ### ######### ######### #########
 ## Implementation
 
-namespace eval ::grammar {}
-namespace eval ::grammar::fa {}
 namespace eval ::grammar::fa::op {
 
     # ### ### ### ######### ######### #########
@@ -38,15 +35,24 @@ namespace eval ::grammar::fa::op {
 
     proc fromRegex   {fa regex {over {}}} {}
 
+    proc toRegexp    {fa} {}
+    proc toRegexp2   {fa} {}
+
+    proc simplifyRegexp {rex} {}
+    proc toTclRegexp    {rex symdict} {}
+
     # ### ### ### ######### ######### #########
 
     namespace export reverse complete remove_eps trim \
 	    determinize minimize complement kleene \
 	    optional union intersect difference \
-	    concatenate fromRegex
+	    concatenate fromRegex toRegexp toRegexp2 \
+	    simplifyRegexp toTclRegexp
 
     # ### ### ### ######### ######### #########
-    ## Internal data structures. None so far.
+    ## Internal data structures.
+
+    variable cons {}
 
     # ### ### ### ######### ######### #########
 }
@@ -276,7 +282,6 @@ proc ::grammar::fa::op::trim {fa {what !reachable|!useful}} {
 # --- --- --- --------- --------- ---------
 
 proc ::grammar::fa::op::determinize {fa {mapvar {}} {idstart 0}} {
-
     # We do the operation in several stages instead of jumping
     # directly in the subset construction. Basically we try the less
     # expensive operations first to see if they are enough. It does
@@ -329,7 +334,7 @@ proc ::grammar::fa::op::determinize {fa {mapvar {}} {idstart 0}} {
     array set subsets {}
     set id      $idstart
     set pending {}
-    set dfa [grammar::fa %AUTO%]
+    set dfa [[cons] %AUTO%]
     # FUTURE : $dfa symbol set [$fa symbols]
     foreach sym [$fa symbols] {$dfa symbol add $sym}
 
@@ -741,14 +746,14 @@ proc ::grammar::fa::op::concatenate {fa fb {mapvar {}}} {
 }
 
 # ### ### ### ######### ######### #########
-## API implementation. Compilation.
+## API implementation. Compilation (regexp -> FA).
 
 proc ::grammar::fa::op::fromRegex {fa regex {over {}}} {
     # Convert a regular expression into a FA. The regex is given as
     # parse tree in the form of a nested list.
 
-    # {. A B ...} ... Concatenation.
-    # {| A B ...} ... Alternatives.
+    # {. A B ...} ... Concatenation (accepts zero|one arguments).
+    # {| A B ...} ... Alternatives  (accepts zero|one arguments).
     # {? A}       ... Optional.
     # {* A}       ... Kleene.
     # {+ A}       ... Pos.Kleene.
@@ -762,7 +767,7 @@ proc ::grammar::fa::op::fromRegex {fa regex {over {}}} {
 	return
     }
 
-    set tmp [::grammar::fa %AUTO%]
+    set tmp [[cons] %AUTO%]
 
     if {![llength $over]} {
 	set over [lsort -uniq [RESymbols $regex]]
@@ -847,7 +852,7 @@ proc ::grammar::fa::op::Regex {fa regex idvar} {
 	    set a $id ; incr id ; $fa state add $a
 	    set b $id ; incr id ; $fa state add $b
 
-	    set tmp [grammar::fa %AUTO%]
+	    set tmp [[cons] %AUTO%]
 	    foreach sym [$fa symbols] {$tmp symbol add $sym}
 	    struct::list assign [Regex $tmp [lindex $regex 1] id] s f
 	    $tmp start add $s
@@ -880,14 +885,14 @@ proc ::grammar::fa::op::Regex {fa regex idvar} {
 		set a $id ; incr id ; $fa state add $a
 		set b $id ; incr id ; $fa state add $b
 
-		set tmp [grammar::fa %AUTO%]
+		set tmp [[cons] %AUTO%]
 		foreach sym [$fa symbols] {$tmp symbol add $sym}
 		set idsub 0
 		struct::list assign [Regex $tmp [lindex $regex 1] idsub] s f
 		$tmp start add $s
 		$tmp final add $f
 
-		set beta [grammar::fa %AUTO%]
+		set beta [[cons] %AUTO%]
 		foreach sub [lrange $regex 2 end] {
 		    foreach sym [$fa symbols] {$beta symbol add $sym}
 		    struct::list assign [Regex $beta $sub idsub] s f
@@ -910,7 +915,16 @@ proc ::grammar::fa::op::Regex {fa regex idvar} {
 	. {
 	    # Concatenation ...
 
-	    if {[llength $regex] < 3} {
+	    if {[llength $regex] == 1} {
+		# Optimized path. No sub-expressions. This represents
+		# language containing only the empty string, aka
+		# epsilon.
+
+		set a $id ; incr id ; $fa state add $a
+		set b $id ; incr id ; $fa state add $b
+		$fa next $a "" --> $b
+
+	    } elseif {[llength $regex] == 2} {
 		# Optimized path. Concatenation of one sub-expression
 		# is the sub-expression itself.
 
@@ -932,7 +946,14 @@ proc ::grammar::fa::op::Regex {fa regex idvar} {
 	| {
 	    # Alternatives ... (Union)
 
-	    if {[llength $regex] < 3} {
+	    if {[llength $regex] == 1} {
+		# Optimized path. No sub-expressions. This represents
+		# the empty language, i.e. the language without words.
+
+		set a $id ; incr id ; $fa state add $a
+		set b $id ; incr id ; $fa state add $b
+
+	    } elseif {[llength $regex] == 2} {
 		# Optimized path. Choice/Union of one sub-expression
 		# is the sub-expression itself.
 
@@ -992,22 +1013,22 @@ proc ::grammar::fa::op::CrossPrepare {fa fb label} {
     set tmp $fb
     set bnew [struct::set difference $totals [$fb symbols]]
     if {[llength $bnew]} {
-	set tmp [grammar::fa %AUTO% = $fb]
+	set tmp [[cons] %AUTO% = $fb]
 	foreach sym $bnew {
 	    $tmp symbol add $sym
 	}    
     }
     if {![$fb is epsilon-free]} {
-	if {$tmp eq $fb} {set tmp [grammar::fa %AUTO% = $fb]}
+	if {$tmp eq $fb} {set tmp [[cons] %AUTO% = $fb]}
 	remove_eps $tmp
 	trim       $tmp
     }
     if {![$fb is complete]} {
-	if {$tmp eq $fb} {set tmp [grammar::fa %AUTO% = $fb]}
+	if {$tmp eq $fb} {set tmp [[cons] %AUTO% = $fb]}
 	complete $tmp
     }
 
-    set res [grammar::fa %AUTO%]
+    set res [[cons] %AUTO%]
     foreach sym $totals {
 	$res symbol add $sym
     } 
@@ -1061,7 +1082,7 @@ proc ::grammar::fa::op::DoCross {fa fb res id seed smapvar} {
 
 # --- --- --- --------- --------- ---------
 
-proc MergePrepare {fa fb label mapvar} {
+proc ::grammar::fa::op::MergePrepare {fa fb label mapvar} {
     upvar 1 $mapvar map
 
     set starta [$fa startstates]
@@ -1075,7 +1096,7 @@ proc MergePrepare {fa fb label mapvar} {
 	return -code error "Unable to $label FAs without start/final states"
     }
 
-    # FUTURE: add {expand}[symbols], ignore dup's
+    # FUTURE: add {*}[symbols], ignore dup's
     foreach sym [$fb symbols] {catch {$fa symbol add $sym}}
 
     set dup [struct::set intersect [$fa states] [$fb states]]
@@ -1090,7 +1111,7 @@ proc MergePrepare {fa fb label mapvar} {
 	# prevent interference between the two.
 
 	set map {}
-	set tmp [grammar::fa %AUTO% = $fb]
+	set tmp [[cons] %AUTO% = $fb]
 	set id 0
 	foreach s $dup {
 	    # The renaming process has to ensure that the new name is
@@ -1149,7 +1170,449 @@ proc ::grammar::fa::op::FindNewState {fa prefix} {
     return ${prefix}.$n
 }
 
+# ### ### ### ######### ######### #########
+## API implementation. Decompilation (FA -> regexp).
+
+proc ::grammar::fa::op::toRegexp {fa} {
+    # NOTE: FUTURE - Do not go through the serialization, nor through
+    # a matrix. The algorithm can be expressed more directly as
+    # operations on the automaton (states and transitions).
+
+    set ET [ser_to_ematrix [$fa serialize]]
+    while {[llength $ET] > 2} {
+	set ET [matrix_drop_state $ET]
+    }
+    return [lindex $ET 0 1]
+}
+
+proc ::grammar::fa::op::toRegexp2 {fa} {
+    # NOTE: FUTURE - See above.
+    set ET [ser_to_ematrix [$fa serialize]]
+    while {[llength $ET] > 2} {
+	set ET [matrix_drop_state $ET re2]
+    }
+    return [lindex $ET 0 1]
+}
+
+# ### ### ### ######### ######### #########
+## Internal helpers.
+
+proc ::grammar::fa::op::ser_to_ematrix {ser} {
+    if {[lindex $ser 0] ne "grammar::fa"} then {
+	error "Expected grammar::fa automaton serialisation"
+    }
+    set stateL {}
+    set n 2; foreach {state des} [lindex $ser 2] {
+	lappend stateL $state
+	set N($state) $n
+	incr n
+    }
+    set row0 {}
+    for {set k 0} {$k<$n} {incr k} {lappend row0 [list |]}
+    set res [list $row0 $row0]
+    foreach {from des} [lindex $ser 2] {
+	set row [lrange $row0 0 1]
+	if {[lindex $des 0]} then {lset res 0 $N($from) [list .]}
+	if {[lindex $des 1]} then {lset row 1 [list .]}
+	foreach to $stateL {set S($to) [list |]}
+	foreach {symbol targetL} [lindex $des 2] {
+	    if {$symbol eq ""} then {
+		set atom [list .]
+	    } else {
+		set atom [list S $symbol]
+	    }
+	    foreach to $targetL {lappend S($to) $atom}
+	}
+	foreach to $stateL {
+	    if {[llength $S($to)] == 2} then {
+		lappend row [lindex $S($to) 1]
+	    } else {
+		lappend row $S($to)
+	    }
+	}
+	lappend res $row
+    }
+    return $res
+}
+
+proc ::grammar::fa::op::matrix_drop_state {T_in {ns re1}} {
+    set sumcmd ${ns}::|
+    set prodcmd ${ns}::.
+    set T1 {}
+    set lastcol {}
+    foreach row $T_in {
+	lappend T1 [lreplace $row end end]
+	lappend lastcol [lindex $row end]
+    }
+    set lastrow [lindex $T1 end]
+    set T1 [lreplace $T1 end end]
+    set b [${ns}::* [lindex $lastcol end]]
+    set lastcol [lreplace $lastcol end end]
+    set res {}
+    foreach row $T1 a $lastcol {
+	set newrow {}
+	foreach pos $row c $lastrow {
+	    lappend newrow [$sumcmd $pos [$prodcmd $a $b $c]]
+	}
+	lappend res $newrow
+    }
+    return $res
+}
+
+# ### ### ### ######### ######### #########
+## Internal helpers. Regexp simplification I.
+
+namespace eval ::grammar::fa::op::re1 {
+    namespace export | . {\*}
+}
+
+proc ::grammar::fa::op::re1::| {args} {
+    set L {}
+
+    # | = Choices.
+    # Sub-choices are lifted into the top expression (foreach).
+    # Identical choices are reduced to a single term (lsort -uniq).
+
+    foreach re $args {
+	switch -- [lindex $re 0] "|" {
+	    foreach term [lrange $re 1 end] {lappend L $term}
+	} default {
+	    lappend L $re
+	}
+    }
+    set L [lsort -unique $L]
+    if {[llength $L] == 1} then {
+	return [lindex $L 0]
+    } else {
+	return [linsert $L 0 |]
+    }
+}
+
+proc ::grammar::fa::op::re1::. {args} {
+    set L {}
+
+    # . = Sequence.
+    # One element sub-choices are lifted into the top expression.
+    # Sub-sequences are lifted into the top expression.
+
+    foreach re $args {
+	switch -- [lindex $re 0] "." {
+	    foreach term [lrange $re 1 end] {lappend L $term}
+	} "|" {
+	    if {[llength $re] == 1} then {return $re}
+	    lappend L $re
+	} default {
+	    lappend L $re
+	}
+    }
+    if {[llength $L] == 1} then {
+	return [lindex $L 0]
+    } else {
+	return [linsert $L 0 .]
+    }
+}
+
+proc ::grammar::fa::op::re1::* {re} {
+    # * = Kleene closure.
+    # Sub-closures are lifted into the top expression.
+    # One-element sub-(choices,sequences) are lifted into the top expression.
+
+    switch -- [lindex $re 0] "|" - "." {
+	if {[llength $re] == 1} then {
+	    return [list .]
+	} else {
+	    return [list * $re]
+	}
+    } "*" {
+	return $re
+    } default {
+	return [list * $re]
+    }
+}
+
+# ### ### ### ######### ######### #########
+## Internal helpers. Regexp simplification II.
+
+namespace eval ::grammar::fa::op::re2 {
+    # Inherit choices and kleene-closure from the basic simplifier.
+
+    namespace import [namespace parent]::re1::|
+    namespace import [namespace parent]::re1::\\*
+}
+
+proc ::grammar::fa::op::re2::. {args} {
+
+    # . = Sequences
+    # Sub-sequences are lifted into the top expression.
+    # Sub-choices are multiplied out.
+    # <Example a(b|c) => ab|ac >
+
+    set L {}
+    set n -1
+    foreach re $args {
+	incr n
+	switch -- [lindex $re 0] "." {
+	    foreach term [lrange $re 1 end] {lappend L $term}
+	} "|" {
+	    set res [list |]
+	    set L2 [lreplace $args 0 $n]
+	    foreach term [lrange $re 1 end] {
+		lappend res [eval [list .] $L [list $term] $L2]
+	    }
+	    return [eval $res]
+	} default {
+	    lappend L $re
+	}
+    }
+    if {[llength $L] == 1} then {
+	return [lindex $L 0]
+    } else {
+	return [linsert $L 0 .]
+    }
+}
+
+# ### ### ### ######### ######### #########
+## API. Simplification of regular expressions.
+
+proc ::grammar::fa::op::simplifyRegexp {RE0} {
+    set RE1 [namespace inscope nonnull $RE0]
+    if {[lindex $RE1 0] eq "S" || $RE1 eq "." || $RE1 eq "|"} then {
+	return $RE1
+    }
+    set tmp [grammar::fa %AUTO% fromRegex $RE1]
+    $tmp minimize
+    set RE1 [toRegexp $tmp]
+    $tmp destroy
+    if {[string length $RE1] < [string length $RE0]} then {
+	set RE0 $RE1
+    }
+    if {[lindex $RE0 0] eq "S"} then {return $RE0}
+    set res [lrange $RE0 0 0]
+    foreach branch [lrange $RE0 1 end] {
+	lappend res [simplifyRegexp $branch]
+    }
+    return $res
+}
+
+# ### ### ### ######### ######### #########
+## Internal helpers.
+
+namespace eval ::grammar::fa::op::nonnull {}
+
+proc ::grammar::fa::op::nonnull::| {args} {
+    set also_empty false
+    set res [list |]
+    foreach branch $args {
+	set RE [eval $branch]
+	if {[lindex $RE 0] eq "?"} then {
+	    set also_empty true
+	    set RE [lindex $RE 1]
+	}
+	switch -- [lindex $RE 0] "|" {
+	    eval [lreplace $RE 0 0 lappend res]
+	} "." {
+	    if {[llength $RE] == 1} then {
+		set also_empty true
+	    } else {
+		lappend res $RE
+	    }
+	} default {
+	    lappend res $RE
+	}
+    }
+    if {!$also_empty} then {return $res}
+    foreach branch [lrange $res 1 end] {
+	if {[lindex $branch 0] eq "*"} then {return $res}
+    }
+    if {[llength $res] == 1} then {
+	return [list .]
+    } elseif {[llength $res] == 2} then {
+	return [lreplace $res 0 0 ?]
+    } else {
+	return [list ? $res]
+    }
+}
+
+proc ::grammar::fa::op::nonnull::. {args} {
+    set res [list .]
+    foreach branch $args {
+	set RE [eval $branch]
+	switch -- [lindex $RE 0] "|" {
+	    if {[llength $RE] == 1} then {return $RE}
+	    lappend res $RE
+	} "." {
+	    eval [lreplace $RE 0 0 lappend res]
+	} default {
+	    lappend res $RE
+	}
+    }
+    return $res
+}
+
+proc ::grammar::fa::op::nonnull::* {sub} {
+    set RE [eval $sub]
+    switch -- [lindex $RE 0] "*" - "?" - "+" {
+	return [lreplace $RE 0 0 *]
+    } default {
+	return [list * $RE]
+    }
+}
+
+proc ::grammar::fa::op::nonnull::+ {sub} {
+    set RE [eval $sub]
+    switch -- [lindex $RE 0] "+" {
+	return $RE
+    } "*" - "?" {
+	return [lreplace $RE 0 0 *]
+    } default {
+	return [list * $RE]
+    }
+}
+
+proc ::grammar::fa::op::nonnull::? {sub} {
+    set RE [eval $sub]
+    switch -- [lindex $RE 0] "?" - "*" {
+	return $RE
+    } "+" {
+	return [lreplace $RE 0 0 *]
+    } default {
+	return [list ? $RE]
+    }
+}
+
+proc ::grammar::fa::op::nonnull::S {name} {
+    return [list S $name]
+}
+
+# ### ### ### ######### ######### #########
+## API. Translate RE of this package to Tcl REs
+
+proc ::grammar::fa::op::toTclRegexp {re symdict} {
+    return [lindex [namespace inscope tclre $re $symdict] 1]
+}
+
+# ### ### ### ######### ######### #########
+## Internal helpers.
+
+namespace eval ::grammar::fa::op::tclre {}
+
+proc ::grammar::fa::op::tclre::S {name dict} {
+    array set A $dict
+    if {[info exists A($name)]} then {
+	return $A($name)
+    } elseif {[string length $name] == 1} then {
+	if {[regexp {[\\\[\]{}.()*+?^$]} $name]} then {
+	    return [list char \\$name]
+	} else {
+	    return [list char $name]
+	}
+    } else {
+	return [list class "\[\[:${name}:\]\]"]
+    }
+}
+
+proc ::grammar::fa::op::tclre::. {args} {
+    set suffix [lrange $args end end]
+    set L {}
+    foreach factor [lrange $args 0 end-1] {
+	set pair [eval $factor $suffix]
+	switch -- [lindex $pair 0] "sum" {
+	    lappend L ([lindex $pair 1])
+	} default {
+	    lappend L [lindex $pair 1]
+	}
+    }
+    return [list prod [join $L ""]]
+}
+
+proc ::grammar::fa::op::tclre::* {re dict} {
+    set pair [eval $re [list $dict]]
+    switch -- [lindex $pair 0] "sum" - "prod" {
+	return [list prod "([lindex $pair 1])*"]
+    } default {
+	return [list prod "[lindex $pair 1]*"]
+    }
+}
+
+proc ::grammar::fa::op::tclre::+ {re dict} {
+    set pair [eval $re [list $dict]]
+    switch -- [lindex $pair 0] "sum" - "prod" {
+	return [list prod "([lindex $pair 1])+"]
+    } default {
+	return [list prod "[lindex $pair 1]+"]
+    }
+}
+
+proc ::grammar::fa::op::tclre::? {re dict} {
+    set pair [eval $re [list $dict]]
+    switch -- [lindex $pair 0] "sum" - "prod" {
+	return [list prod "([lindex $pair 1])?"]
+    } default {
+	return [list prod "[lindex $pair 1]?"]
+    }
+}
+
+proc ::grammar::fa::op::tclre::| {args} {
+    set suffix [lrange $args end end]
+    set charL {}
+    set classL {}
+    set prodL {}
+    foreach factor [lrange $args 0 end-1] {
+	set pair [eval $factor $suffix]
+	switch -- [lindex $pair 0] "char" {
+	    lappend charL [lindex $pair 1]
+	} "class" {
+	    lappend classL [string range [lindex $pair 1] 1 end-1]
+	} default {
+	    lappend prodL [lindex $pair 1]
+	}
+    }
+    if {[llength $charL]>1 || [llength $classL]>0} then {
+	while {[set n [lsearch $charL -]] >= 0} {
+	    lset charL $n {\-}
+	}
+	set bracket "\[[join $charL ""][join $classL ""]\]"
+	if {![llength $prodL]} then {
+	    return [list atom $bracket]
+	}
+	lappend prodL $bracket
+    } else {
+	eval [list lappend prodL] $charL
+    }
+    return [list sum [join $prodL |]]
+}
+
+proc ::grammar::fa::op::tclre::& {args} {
+    error "Cannot express language intersection in Tcl-RE's"
+
+    # Note: This can be translated by constructing an automaton for
+    # the intersection, and then translating its conversion to a
+    # regular expression.
+}
+
+proc ::grammar::fa::op::tclre::! {args} {
+    error "Cannot express language complementation in Tcl-RE's"
+
+    # Note: This can be translated by constructing an automaton for
+    # the complement, and then translating its conversion to a regular
+    # expression. This however requires knowledge regarding the set of
+    # symbols. Large (utf-8) for Tcl regexes.
+}
+
+# ### ### ### ######### ######### #########
+
+proc ::grammar::fa::op::constructor {cmd} {
+    variable cons $cmd
+    return
+}
+
+proc ::grammar::fa::op::cons {} {
+    variable cons
+    if {$cons ne ""} {return $cons}
+    return -code errror "No constructor for FA container was established."
+}
+
 # ### ### ### ######### ######### #########
 ## Package Management
 
-package provide grammar::fa::op 0.1.1
+package provide grammar::fa::op 0.4
