@@ -8,13 +8,15 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# RCS: @(#) $Id$
+# RCS: @(#) $Id: math.tcl,v 1.22 2006/09/20 22:16:02 andreas_kupries Exp $
 
 package require Tcl 8.2		;# uses [lindex $l end-$integer]
 
+# @mdgen OWNER: tclIndex
+
 namespace eval ::math {
 
-    variable version 1.2.2
+    variable version 1.2.4
 
     # misc.tcl
 
@@ -22,7 +24,7 @@ namespace eval ::math {
     namespace export	max		mean		min
     namespace export	product		random		sigma
     namespace export	stats		sum
-    namespace export	expectDouble
+    namespace export	expectDouble    expectInteger
 
     # combinatorics.tcl
 
