@@ -8,14 +8,18 @@
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # -------------------------------------------------------------------------
 #
-# $Id$
+# $Id: md4.tcl,v 1.19 2006/09/19 23:36:17 andreas_kupries Exp $
 
 package require Tcl 8.2;                # tcl minimum version
 catch {package require md4c 1.0};       # tcllib critcl alternative
 
+# @mdgen EXCLUDE: md4c.tcl
+
 namespace eval ::md4 {
-    variable version 1.0.2
-    variable rcsid {$Id$}
+    variable version 1.0.4
+    variable rcsid {$Id: md4.tcl,v 1.19 2006/09/19 23:36:17 andreas_kupries Exp $}
+    variable accel
+    array set accel {critcl 0 cryptkit 0}
 
     namespace export md4 hmac MD4Init MD4Update MD4Final
 
@@ -32,32 +36,40 @@ namespace eval ::md4 {
 #
 proc ::md4::MD4Init {} {
     variable uid
+    variable accel
     set token [namespace current]::[incr uid]
-    upvar #0 $token tok
+    upvar #0 $token state
 
     # RFC1320:3.3 - Initialize MD4 state structure
-    array set tok \
+    array set state \
         [list \
              A [expr {0x67452301}] \
              B [expr {0xefcdab89}] \
              C [expr {0x98badcfe}] \
              D [expr {0x10325476}] \
              n 0 i "" ]
+    if {$accel(cryptkit)} {
+        cryptkit::cryptCreateContext state(ckctx) CRYPT_UNUSED CRYPT_ALGO_MD4
+    }
     return $token
 }
 
 proc ::md4::MD4Update {token data} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    variable accel
+    upvar #0 $token state
 
-    if {[package provide md4c] != {}} {
+    if {$accel(critcl)} {
         if {[info exists state(md4c)]} {
             set state(md4c) [md4c $data $state(md4c)]
         } else {
             set state(md4c) [md4c $data]
         }
         return
+    } elseif {[info exists state(ckctx)]} {
+        if {[string length $data] > 0} {
+            cryptkit::cryptEncrypt $state(ckctx) $data
+        }
+        return
     }
 
     # Update the state values
@@ -76,14 +88,22 @@ proc ::md4::MD4Update {token data} {
 }
 
 proc ::md4::MD4Final {token} {
-    # FRINK: nocheck
-    variable $token
-    upvar 0 $token state
+    upvar #0 $token state
 
-    if {[package provide md4c] != {}} {
+    if {[info exists state(md4c)]} {
         set r $state(md4c)
         unset state
         return $r
+    } elseif {[info exists state(ckctx)]} {
+        cryptkit::cryptEncrypt $state(ckctx) ""
+        cryptkit::cryptGetAttributeString $state(ckctx) \
+            CRYPT_CTXINFO_HASHVALUE r 16
+        cryptkit::cryptDestroyContext $state(ckctx)
+        # If nothing was hashed, we get no r variable set!
+        if {[info exists r]} {
+            unset state
+            return $r
+        }
     }
 
     # RFC1320:3.1 - Padding
@@ -286,7 +306,7 @@ proc ::md4::<<< {v n} {
                       & 0xFFFFFFFF}]
 }
 
-# Convert our <<< pseuodo-operator into a procedure call.
+# Convert our <<< pseudo-operator into a procedure call.
 regsub -all -line \
     {\[expr {(.*) <<< (\d+)}\]} \
     $::md4::MD4Hash_body \
@@ -338,17 +358,51 @@ if {[package provide Trf] != {}} {
     interp alias {} ::md4::Hex {} ::hex -mode encode --
 } else {
     proc ::md4::Hex {data} {
-        set result {}
-        binary scan $data c* r
-        foreach c $r {
-            append result [format "%02X" [expr {$c & 0xff}]]
-        }
-        return $result
+        binary scan $data H* result
+        return [string toupper $result]
     }
 }
 
 # -------------------------------------------------------------------------
 
+# LoadAccelerator --
+#
+#	This package can make use of a number of compiled extensions to
+#	accelerate the digest computation. This procedure manages the
+#	use of these extensions within the package. During normal usage
+#	this should not be called, but the test package manipulates the
+#	list of enabled accelerators.
+#
+proc ::md4::LoadAccelerator {name} {
+    variable accel
+    set r 0
+    switch -exact -- $name {
+        critcl {
+            if {![catch {package require tcllibc}]
+                || ![catch {package require md4c}]} {
+                set r [expr {[info command ::md4::md4c] != {}}]
+            }
+        }
+        cryptkit {
+            if {![catch {package require cryptkit}]} {
+                set r [expr {![catch {cryptkit::cryptInit}]}]
+            }
+        }
+        #trf {
+        #    if {![catch {package require Trf}]} {
+        #        set r [expr {![catch {::md4 aa} msg]}]
+        #    }
+        #}
+        default {
+            return -code error "invalid accelerator package:\
+                must be one of [join [array names accel] {, }]"
+        }
+    }
+    set accel($name) $r
+}
+
+# -------------------------------------------------------------------------
+
 # Description:
 #  Pop the nth element off a list. Used in options processing.
 #
@@ -386,8 +440,9 @@ proc ::md4::md4 {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0 } { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
                     must be one of $err"
@@ -437,7 +492,7 @@ proc ::md4::md4 {args} {
 # -------------------------------------------------------------------------
 
 proc ::md4::hmac {args} {
-    array set opts {-hex 0 -filename {} -channel {} -chunksize 4096 -key {}}
+    array set opts {-hex 0 -filename {} -channel {} -chunksize 4096}
     while {[string match -* [set option [lindex $args 0]]]} {
         switch -glob -- $option {
             -key       { set opts(-key) [Pop args 1] }
@@ -445,8 +500,9 @@ proc ::md4::hmac {args} {
             -file*     { set opts(-filename) [Pop args 1] }
             -channel   { set opts(-channel) [Pop args 1] }
             -chunksize { set opts(-chunksize) [Pop args 1] }
-            --         { Pop args ; break }
             default {
+                if {[llength $args] == 1} { break }
+                if {[string compare $option "--"] == 0 } { Pop args; break }
                 set err [join [lsort [array names opts]] ", "]
                 return -code error "bad option $option:\
                     must be one of $err"
@@ -455,7 +511,7 @@ proc ::md4::hmac {args} {
         Pop args
     }
 
-    if {$opts(-key) == {}} {
+    if {![info exists opts(-key)]} {
         return -code error "wrong # args:\
             should be \"hmac ?-hex? -key key -filename file | string\""
     }
@@ -500,6 +556,11 @@ proc ::md4::hmac {args} {
 
 # -------------------------------------------------------------------------
 
+# Try and load a compiled extension to help.
+namespace eval ::md4 {
+    foreach e {critcl cryptkit} { if {[LoadAccelerator $e]} { break } }
+}
+
 package provide md4 $::md4::version
 
 # -------------------------------------------------------------------------
