@@ -33,7 +33,7 @@ proc critcl_notes {text} {
     return
 }
 
-source [file join $distribution package_version.tcl] ; # Get version information.
+source [file join $distribution support installation version.tcl] ; # Get version information.
 
 set package_nv ${package_name}-${package_version}
 
@@ -96,7 +96,6 @@ proc modtclfiles {modules} {
     return $mfiles
 }
 
-
 proc modules {} {
     global distribution
     set fl [list]
@@ -117,10 +116,21 @@ proc modules_mod {m} {
     return [expr {[lsearch -exact [modules] $m] >= 0}]
 }
 
+proc dealias {modules} {
+    set _ {}
+    foreach m $modules {
+	if {[file exists $m]} {
+	    set m [file tail $m]
+	}
+	lappend _ $m
+    }
+    return $_
+}
+
 proc load_modinfo {} {
     global distribution modules guide
-    source [file join $distribution installed_modules.tcl] ; # Get list of installed modules.
-    source [file join $distribution install_action.tcl]    ; # Get installer support code.
+    source [file join $distribution support installation modules.tcl] ; # Get list of installed modules.
+    source [file join $distribution support installation actions.tcl] ; # Get installer support code.
     proc load_modinfo {} {}
     return
 }
@@ -239,6 +249,9 @@ proc ppackages {args} {
 	    if {![regexp {provide} $line]} {continue}
 	    if {![regexp {package} $line]} {continue}
 
+	    # Now a stronger check for the actual command
+	    if {![regexp {package[ 	][ 	]*provide} $line]} {continue}
+
 	    set xline $line
 	    regsub {^.*provide } $line {} line
 	    regsub {\].*$}       $line {\1} line
@@ -247,6 +260,15 @@ proc ppackages {args} {
 
 	    foreach {n v} $line break
 
+	    # HACK ...
+	    # Module 'page', package 'page::gen::peg::cpkg'.
+	    # Has a provide statement inside a template codeblock.
+	    # Name is placeholder @@. Ignore this specific name.
+	    # Better would be to use general static Tcl parsing
+	    # to find that the string is a variable value.
+
+	    if {[string equal $n @@]} continue
+
 	    if {[regexp {^[0-9]+(\.[0-9]+)*$} $v]} {
 		lappend p($n) $v
 		set p($n) [lsort -uniq -dict $p($n)]
@@ -368,70 +390,6 @@ proc xPackage {cmd args} {
 
 proc sep {} {puts ~~~~~~~~~~~~~~~~~~~~~~~~}
 
-proc gendoc {fmt ext args} {
-    global distribution
-    global tcl_platform
-
-    set null   0
-    set hidden 0
-    if {![string compare $fmt null]} {set null   1}
-    if {![string compare $fmt desc]} {set hidden 1}
-    if {[llength $args] == 0} {set args [modules]}
-
-    if {!$null} {
-	file mkdir [file join doc $fmt]
-    }
-
-    getpackage doctools doctools/doctools.tcl
-
-    foreach m $args {
-	::doctools::new dt \
-		-format $fmt \
-		-module $m
-
-	set fl [glob -nocomplain [file join $distribution modules $m *.man]]
-
-	if {[llength $fl] == 0} {
-	    dt destroy
-	    continue
-	}
-
-	foreach f $fl {
-	    if {!$null} {
-                set target [file join doc $fmt \
-                                [file rootname [file tail $f]].$ext]
-                if {[file exists $target] 
-                    && [file mtime $target] > [file mtime $f]} {
-                    continue
-                }
-	    }
-	    if {!$hidden} {puts "Gen ($fmt): $f"}
-
-	    dt configure -file $f
-	    if {$null} {
-		dt configure -deprecated 1
-	    }
-
-	    if {[catch {
-		set data [dt format [get_input $f]]
-	    } msg]} {
-		puts $msg
-		continue
-	    }
-
-	    set warnings [dt warnings]
-	    if {[llength $warnings] > 0} {
-		puts stderr [join $warnings \n]
-	    }
-
-	    if {!$null} {
-		write_out $target $data
-	    }
-	}
-	dt destroy
-    }
-}
-
 proc gd-cleanup {} {
     global package_nv
 
@@ -474,6 +432,8 @@ proc gd-gen-archives {} {
 
     set sdx [auto_execok sdx]
     if {$sdx != {}} {
+	file copy -force [file join ${package_nv} support installation main.tcl] \
+		[file join ${package_nv} main.tcl]
 	file rename ${package_nv} ${package_name}.vfs
 
 	puts "    Starkit         (${package_nv}.kit)..."
@@ -563,6 +523,15 @@ proc gd-assemble {} {
     return
 }
 
+proc normalize-version {v} {
+    # Strip everything after the first non-version character, and any
+    # trailing dots left behind by that, to avoid the insertion of bad
+    # version numbers into the generated .tap file.
+
+    regsub {[^0-9.].*$} $v {} v
+    return [string trimright $v .]
+}
+
 proc gd-gen-tap {} {
     getpackage textutil textutil/textutil.tcl
     getpackage fileutil fileutil/fileutil.tcl
@@ -596,7 +565,7 @@ proc gd-gen-tap {} {
     lappend lines {# ###############}
     lappend lines {# Complete bundle}
     lappend lines {}
-    lappend lines [list Package [list $package_name $package_version]]
+    lappend lines [list Package [list $package_name [normalize-version $package_version]]]
     lappend lines "Base     @TAP_DIR@"
     lappend lines "Platform *"
     lappend lines "Desc     \{$pname: Bundle of all packages\}"
@@ -658,7 +627,7 @@ proc gd-gen-tap {} {
 
 		foreach v $vlist {
 		    lappend lines {}
-		    lappend lines [list Package [list $p $v]]
+		    lappend lines [list Package [list $p [normalize-version $v]]]
 		    lappend lines "See   [list __$m]"
 		    lappend lines "Platform *"
 		    lappend lines "Desc     \{$desc\}"
@@ -677,7 +646,7 @@ proc gd-gen-tap {} {
 
 	    lappend lines "# -------+"
 	    lappend lines {}
-	    lappend lines [list Package [list $p $v]]
+	    lappend lines [list Package [list $p [normalize-version $v]]]
 	    lappend lines "Platform *"
 	    lappend lines "Desc     \{$desc\}"
 	    lappend lines "Base     @TAP_DIR@/$m"
@@ -708,7 +677,8 @@ proc gd-gen-tap {} {
 proc getpdesc  {} {
     global argv ; if {![checkmod]} return
 
-    eval gendoc desc l $argv
+    package require sak::doc
+    sak::doc::Gen desc l $argv
     
     array set _ {}
     foreach file [glob -nocomplain doc/desc/*.l] {
@@ -727,7 +697,7 @@ proc getpdesc  {} {
 proc gd-gen-rpmspec {} {
     global package_version package_name distribution
 
-    set in  [file join $distribution package_rpm.txt]
+    set in  [file join $distribution support releases package_rpm.txt]
     set out [file join $distribution ${package_name}.spec]
 
     write_out $out [string map \
@@ -744,7 +714,7 @@ proc gd-gen-yml {} {
 
     global package_version package_name distribution
 
-    set in  [file join $distribution package_yml.txt]
+    set in  [file join $distribution support releases package_yml.txt]
     set out [file join $distribution ${package_name}.yml]
 
     write_out $out [string map \
@@ -774,7 +744,7 @@ proc gd-tip55 {} {
     global package_version package_name distribution contributors
     contributors
 
-    set in  [file join $distribution package_tip55.txt]
+    set in  [file join $distribution support releases package_tip55.txt]
     set out [file join $distribution DESCRIPTION.txt]
 
     set md [string map \
@@ -907,26 +877,6 @@ proc validate_versions_mod {m} {
     return
 }
 
-proc timing_mod {m} {
-    global distribution
-    set files [glob -nocomplain [file join $distribution modules $m *.timing]]
-    foreach file $files {
-        interp create slave
-        slave eval [list source $file]
-        interp delete slave
-    }
-    
-    return
-}
-
-proc timing_all {} {
-    global distribution
-    foreach m [modules] {
-	timing_mod $m
-    }
-    return
-}
-
 proc validate_testsuite_mod {m} {
     global distribution
     if {[llength [glob -nocomplain [file join $distribution modules $m *.test]]] == 0} {
@@ -935,6 +885,88 @@ proc validate_testsuite_mod {m} {
     return
 }
 
+proc bench_mod {mlist paths interp flags norm format verbose output} {
+    global distribution env tcl_platform
+
+    getpackage logger logger/logger.tcl
+    getpackage bench  bench/bench.tcl
+
+    ::logger::setlevel $verbose
+
+    set pattern tclsh*
+    if {$interp != {}} {
+	set pattern [file tail $interp]
+	set paths [list [file dirname $interp]]
+    } elseif {![llength $paths]} {
+	# Using the environment PATH is not a good default for
+	# SAK. Use the interpreter running SAK as the default.
+	if 0 {
+	    set paths [split $env(PATH) \
+			   [expr {($tcl_platform(platform) == "windows") ? ";" : ":"}]]
+	}
+	set interp [info nameofexecutable]
+	set pattern [file tail $interp]
+	set paths [list [file dirname $interp]]
+    }
+
+    set interps [bench::versions \
+	    [bench::locate $pattern $paths]]
+
+    if {![llength $interps]} {
+	puts "No interpreters found"
+	return
+    }
+
+    if {[llength $flags]} {
+	set cmd [linsert $flags 0 bench::run]
+    } else {
+	set cmd [list bench::run]
+    }
+
+    array set DATA {}
+
+    foreach m $mlist {
+	set files [glob -nocomplain [file join $distribution modules $m *.bench]]
+	if {![llength $files]} {
+	    bench::log::warn "No benchmark files found for module \"$m\""
+	    continue
+	}
+
+	set run $cmd
+	lappend run $interps $files
+	array set DATA [eval $run]
+    }
+
+    _bench_write $output [array get DATA] $norm $format
+    return
+}
+
+proc bench_all {flags norm format verbose output} {
+    bench_mod [modules] $flags $norm $format $verbose $output
+    return
+}
+
+
+proc _bench_write {output data norm format} {
+    if {$norm != {}} {
+	getpackage logger logger/logger.tcl
+	getpackage bench  bench/bench.tcl
+
+	set data [bench::norm $data $norm]
+    }
+
+    set data [bench::out::$format $data]
+
+    if {$output == {}} {
+	puts $data
+    } else {
+	set    output [open $output w]
+	puts  $output "# -*- tcl -*- bench/$format"
+	puts  $output $data
+	close $output
+    }
+}
+
 proc validate_testsuites {} {
     foreach m [modules] {
 	validate_testsuite_mod $m
@@ -986,24 +1018,14 @@ proc validate_doc_existence {} {
 
 
 proc validate_doc_markup_mod {m} {
-    gendoc null null $m
+    package require sak::doc
+    sak::doc::Gen null null [list $m]
     return
 }
 
 proc validate_doc_markup {} {
-    gendoc null null
-    return
-}
-
-proc timing_mod {m} {
-    global distribution
-    set files [glob -nocomplain [file join $distribution modules $m *.timing]]
-    foreach file $files {
-        interp create slave
-        slave eval [list source $file]
-        interp delete slave
-    }
-    
+    package require sak::doc
+    sak::doc::Gen null null [modules]
     return
 }
 
@@ -1071,6 +1093,26 @@ proc run-tclchecker {args} {
     return
 }
 
+proc run-nagelfar {args} {
+    global distribution
+
+    if {[llength $args] == 0} {
+	set files [tclfiles]
+    } else {
+	set files [lsort -dict [modtclfiles $args]]
+    }
+
+    foreach f $files {
+	puts "NAGELFAR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+	puts "$f ..."
+	puts "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"
+
+	catch {exec nagelfar >@ stdout $f}
+    }
+    return
+}
+
+
 proc get_input {f} {return [read [set if [open $f r]]][close $if]}
 
 proc write_out {f text} {
@@ -1079,10 +1121,15 @@ proc write_out {f text} {
     close $of
 }
 
+proc location_PACKAGES {} {
+    global distribution
+    return [file join $distribution support releases PACKAGES]
+}
+
 proc gd-gen-packages {} {
     global package_version distribution
 
-    set P [file join $distribution PACKAGES]
+    set P [location_PACKAGES]
     file copy -force $P $P.LAST
     set f [open $P w]
     puts $f "@@ RELEASE $package_version"
@@ -1392,97 +1439,6 @@ proc ::dsrs::Final {} {
     }
 }
 
-# --------------------------------------------------------------
-# Help
-
-proc __help {} {
-    global critcldefault
-
-    puts stdout [string map [list @@ $critcldefault] {
-	Commands avalable through the swiss army knife aka SAK:
-
-	help     - This help
-
-	/Configuration
-	/===========================================================
-
-	version  - Return the bundle's version number
-	major    - Return the bundle's major version number
-	minor    - Return the bundle's minor version number
-	name     - Return the bundle's package name
-
-	/Development
-	/===========================================================
-
-	modules          - Return list of modules.
-        contributors     - Print a list of contributors to the bundle.
-	lmodules         - See above, however one module per line
-	imodules         - Return list of modules known to the installer.
-        critcl-modules   - Return a list of modules with critcl enhancements.
-
-	packages         - Return indexed packages in the bundle, plus versions,
-	                   one package per line. Extracted from the
-	                   package indices found in the modules.
-	provided         - Return list and versions of provided packages
-	                   (in contrast to indexed).
-	vcompare pkglist - Compare package list of previous 'packages'
-	                   call with current packages. Marks all new
-	                   and unchanged packages for higher attention.
-
-        critcl ?module?  - Build a critcl module [default is @@].
-
-        timing ?module?  - Run timing scripts (*.timing).
-
-        validate ?module..?     - Check listed modules for problems.
-                                  For all modules if none specified.
-
-        validate_v ?module..?   - Check listed modules for for version
-                                  problems. For all modules if none
-                                  specified.
-
-	test ?module...?        - Run testsuite for listed modules.
-	                          For all modules if none specified.
-
-        docstrip/users             - List modules using docstrip
-        docstrip/regen ?module...? - Regenerate the sources of all
-                                     or the listed modules from their
-                                     docstrip sources.
-
-	/Documentation
-	/===========================================================
-
-	nroff ?module...?    - Generate manpages
-	html  ?module...?    - Generate HTML pages
-	tmml  ?module...?    - Generate TMML
-	text  ?module...?    - Generate plain text
-	list  ?module...?    - Generate a list of manpages
-	wiki  ?module...?    - Generate wiki markup
-	latex ?module...?    - Generate LaTeX pages
-	dvi   ?module...?    - See latex, + conversion to dvi
-	ps    ?module...?    - See dvi,   + conversion to PostScript
-
-	desc  ?module...?    - Module/Package descriptions
-	desc/2 ?module...?   - Module/Package descriptions, alternate format.
-
-	/Release engineering
-	/===========================================================
-
-	gendist  - Generate distribution from CVS snapshot
-
-	rpmspec  - Generate a RPM spec file for the bundle.
-        gentip55 - Generate a TIP55-style DESCRIPTION.txt file.
-        yml      - Generate a YAML description file.
-
-	release name sf-user-id
-	         - Marks the current state of all files as a new
-	           release. This updates all ChangeLog's, and
-	           regenerates the contents of PACKAGES
-
-	rstatus  - Determines the status of the code base with regard
-	           to the last release.
-    }]
-}
-
 # --------------------------------------------------------------
 # Configuration
 
@@ -1553,7 +1509,7 @@ proc __rstatus {} {
 	}
 	close $f
     }
-    pkg-compare [file join $distribution PACKAGES]
+    pkg-compare [location_PACKAGES]
     return
 }
 
@@ -1683,40 +1639,10 @@ proc pkg-compare {oldplist} {
     return
 }
 
-
-
-proc __test {} {
-    global argv distribution
-    # Run testsuite
-
-    set modules $argv
-    if {[llength $modules] == 0} {
-	set modules [modules]
-    }
-
-    puts "Shell is \"[info nameofexecutable]\""
-
-    exec [info nameofexecutable] \
-	    [file join $distribution all.tcl] \
-	    -modules $modules \
-	    >@ stdout 2>@ stderr
-    return
-}
-
 proc checkmod {} {
     global argv
-    set fail 0
-    foreach m $argv {
-	if {![modules_mod $m]} {
-	    puts "  Bogus module: $m"
-	    set fail 1
-	}
-    }
-    if {$fail} {
-	puts "  Stop."
-	return 0
-    }
-    return 1
+    package require sak::util
+    return [sak::util::checkModules argv]
 }
 
 # -------------------------------------------------------------------------
@@ -1801,7 +1727,7 @@ proc __critcl {} {
 
             critcl_module $critcldefault $flags
         } else {
-            foreach m $argv {
+            foreach m [dealias $argv] {
                 if {[info exists critclmodules($m)]} {
                     critcl_module $m $flags
                 } else {
@@ -1858,42 +1784,272 @@ proc critcl_module {pkg {extra ""}} {
 
 # -------------------------------------------------------------------------
 
-proc __timing {} {
+proc __bench/edit {} {
+    global argv argv0
+
+    set format text
+    set output {}
+
+    while {[string match -* [set option [lindex $argv 0]]]} {
+	set val [lindex $argv 1]
+        switch -exact -- $option {
+	    -format {
+		switch -exact -- $val {
+		    raw - csv - text {}
+		    default {
+			return -error "Bad format \"$val\", expected text, csv, or raw"
+		    }
+		}
+		set format $val
+	    }
+	    -o    {set output $val}
+            -- {
+		set argv [lrange $argv 1 end]
+		break
+	    }
+            default { break }
+        }
+        set argv [lrange $argv 2 end]
+    }
+
+    switch -exact -- $format {
+	raw {}
+	csv {
+	    getpackage csv             csv/csv.tcl
+	    getpackage bench::out::csv bench/bench_wcsv.tcl
+	}
+	text {
+	    getpackage report           report/report.tcl
+	    getpackage struct::matrix   struct/matrix.tcl
+	    getpackage bench::out::text bench/bench_wtext.tcl
+	}
+    }
+
+    getpackage bench::in bench/bench_read.tcl
+    getpackage bench     bench/bench.tcl
+
+    if {[llength $argv] != 3} {
+	puts "Usage: $argv0 benchdata column newvalue"
+    }
+
+    foreach {in col new} $argv break
+
+    _bench_write $output \
+	[bench::edit \
+	     [bench::in::read $in] \
+	     $col $new] \
+	{} $format
+    return
+}
+
+proc __bench/del {} {
+    global argv argv0
+
+    set format text
+    set output {}
+
+    while {[string match -* [set option [lindex $argv 0]]]} {
+	set val [lindex $argv 1]
+        switch -exact -- $option {
+	    -format {
+		switch -exact -- $val {
+		    raw - csv - text {}
+		    default {
+			return -error "Bad format \"$val\", expected text, csv, or raw"
+		    }
+		}
+		set format $val
+	    }
+	    -o    {set output $val}
+            -- {
+		set argv [lrange $argv 1 end]
+		break
+	    }
+            default { break }
+        }
+        set argv [lrange $argv 2 end]
+    }
+
+    switch -exact -- $format {
+	raw {}
+	csv {
+	    getpackage csv             csv/csv.tcl
+	    getpackage bench::out::csv bench/bench_wcsv.tcl
+	}
+	text {
+	    getpackage report           report/report.tcl
+	    getpackage struct::matrix   struct/matrix.tcl
+	    getpackage bench::out::text bench/bench_wtext.tcl
+	}
+    }
+
+    getpackage bench::in bench/bench_read.tcl
+    getpackage bench     bench/bench.tcl
+
+    if {[llength $argv] < 2} {
+	puts "Usage: $argv0 benchdata column..."
+    }
+
+    set in [lindex $argv 0]
+
+    set data [bench::in::read $in]
+
+    foreach c [lrange $argv 1 end] {
+	set data [bench::del $data $c]
+    }
+
+    _bench_write $output $data {} $format
+    return
+}
+
+proc __bench/show {} {
     global argv
+
+    set format text
+    set output {}
+    set norm   {}
+
+    while {[string match -* [set option [lindex $argv 0]]]} {
+	set val [lindex $argv 1]
+        switch -exact -- $option {
+	    -format {
+		switch -exact -- $val {
+		    raw - csv - text {}
+		    default {
+			return -error "Bad format \"$val\", expected text, csv, or raw"
+		    }
+		}
+		set format $val
+	    }
+	    -o    {set output $val}
+	    -norm {set norm $val}
+            -- {
+		set argv [lrange $argv 1 end]
+		break
+	    }
+            default { break }
+        }
+        set argv [lrange $argv 2 end]
+    }
+
+    switch -exact -- $format {
+	raw {}
+	csv {
+	    getpackage csv             csv/csv.tcl
+	    getpackage bench::out::csv bench/bench_wcsv.tcl
+	}
+	text {
+	    getpackage report           report/report.tcl
+	    getpackage struct::matrix   struct/matrix.tcl
+	    getpackage bench::out::text bench/bench_wtext.tcl
+	}
+    }
+
+    getpackage bench::in bench/bench_read.tcl
+
+    array set DATA {}
+
+    foreach path $argv {
+	array set DATA [bench::in::read $path]
+    }
+
+    _bench_write $output [array get DATA] $norm $format
+    return
+}
+
+proc __bench {} {
+    global argv
+
+    # I. Process command line arguments for the
+    #    benchmark commands - Validation, possible
+    #    translation ...
+
+    set flags   {}
+    set norm    {}
+    set format  text
+    set verbose warn
+    set output  {}
+    set paths   {}
+    set interp  {}
+
+    while {[string match -* [set option [lindex $argv 0]]]} {
+	set val [lindex $argv 1]
+        switch -exact -- $option {
+	    -throwerrors {lappend flags -errors $val}
+	    -match -
+	    -rmatch -
+	    -iters -
+	    -threads {lappend flags $option $val}
+	    -o       {set output $val}
+	    -norm    {set norm $val}
+	    -path    {lappend paths $val}
+	    -interp  {set interp $val}
+	    -format  {
+		switch -exact -- $val {
+		    raw - csv - text {}
+		    default {
+			return -error "Bad format \"$val\", expected text, csv, or raw"
+		    }
+		}
+		set format $val
+	    }
+	    -verbose {
+		set verbose info
+		set argv [lrange $argv 1 end]
+		continue
+	    }
+	    -debug {
+		set verbose debug
+		set argv [lrange $argv 1 end]
+		continue
+	    }
+            -- {
+		set argv [lrange $argv 1 end]
+		break
+	    }
+            default { break }
+        }
+        set argv [lrange $argv 2 end]
+    }
+
+    switch -exact -- $format {
+	raw {}
+	csv {
+	    getpackage csv             csv/csv.tcl
+	    getpackage bench::out::csv bench/bench_wcsv.tcl
+	}
+	text {
+	    getpackage report           report/report.tcl
+	    getpackage struct::matrix   struct/matrix.tcl
+	    getpackage bench::out::text bench/bench_wtext.tcl
+	}
+    }
+
+    # Choose between benchmarking everything, or
+    # only selected modules.
+
     if {[llength $argv] == 0} {
-	_timing_all
+	_bench_all $paths $interp $flags $norm $format $verbose $output
     } else {
 	if {![checkmod]} {return}
-	foreach m $argv {
-	    _timing_module $m
-	}
+	_bench_module [dealias $argv] $paths $interp $flags $norm $format $verbose $output
     }
     return
 }
 
-proc _timing_module {m} {
+proc _bench_module {mlist paths interp flags norm format verbose output} {
     global package_name package_version
 
-    puts "Timing $package_name $package_version development -- $m"
+    puts "Benchmarking $package_name $package_version development"
     puts "======================================================"
-    timing_mod $m
+    bench_mod $mlist $paths $interp $flags $norm $format $verbose $output
     puts "------------------------------------------------------"
     puts ""
     return
 }
 
-proc _timing_all {} {
-    global package_name package_version distribution
-
-    foreach m [modules] {
-        if {[llength [glob -nocomplain [file join $distribution modules $m *.timing]]] > 0} {
-            puts "Timing $package_name $package_version development -- $m"
-            puts "======================================================"
-            timing_mod $m
-            puts "------------------------------------------------------"
-            puts ""
-        }
-    }
+proc _bench_all {paths flags interp norm format verbose output} {
+    _bench_module [modules] $paths $interp $flags $norm $format $verbose $output
     return
 }
 
@@ -1905,7 +2061,7 @@ proc __validate_v {} {
 	_validate_all_v
     } else {
 	if {![checkmod]} {return}
-	foreach m $argv {
+	foreach m [dealias $argv] {
 	    _validate_module_v $m
 	}
     }
@@ -2002,15 +2158,20 @@ proc _validate_all {} {
     set frink      [auto_execok frink]
     set procheck   [auto_execok procheck]
     set tclchecker [auto_execok tclchecker]
+    set nagelfar [auto_execok nagelfar]
 
     if {$frink == {}} {puts "  Tool 'frink'    not found, no check"}
     if {($procheck == {}) || ($tclchecker == {})} {
 	puts "  Tools 'procheck'/'tclchecker' not found, no check"
     }
-    if {($frink == {}) || ($procheck == {}) || ($tclchecker == {})} {
+    if {$nagelfar == {}} {puts "  Tool 'nagelfar' not found, no check"}
+
+    if {($frink == {}) || ($procheck == {}) || ($tclchecker == {}) 
+        || ($nagelfar == {})} {
 	puts "------------------------------------------------------"
     }
-    if {($frink == {}) && ($procheck == {}) && ($tclchecker == {})} {
+    if {($frink == {}) && ($procheck == {}) && ($tclchecker == {})
+        && ($nagelfar == {})} {
 	return
     }
     if {$frink != {}} {
@@ -2024,6 +2185,10 @@ proc _validate_all {} {
 	run-procheck
 	puts "------------------------------------------------------"
     }
+    if {$nagelfar    !={}} {
+    	run-nagelfar 
+	puts "------------------------------------------------------"
+    }
     puts ""
     return
 }
@@ -2075,23 +2240,38 @@ proc _validate_module {m} {
 
     set frink    [auto_execok frink]
     set procheck [auto_execok procheck]
-
+    set nagelfar [auto_execok nagelfar]
+    set tclchecker [auto_execok tclchecker]
+    
     if {$frink    == {}} {puts "  Tool 'frink'    not found, no check"}
-    if {$procheck == {}} {puts "  Tool 'procheck' not found, no check"}
-    if {($frink == {}) || ($procheck == {})} {
+    if {($procheck == {}) || ($tclchecker == {})} {
+	puts "  Tools 'procheck'/'tclchecker' not found, no check"
+    }
+    if {$nagelfar == {}} {puts "  Tool 'nagelfar' not found, no check"}
+    
+    if {($frink == {}) || ($procheck == {}) || ($tclchecker == {}) ||
+    	($nagelfar == {})} {
 	puts "------------------------------------------------------"
     }
-    if {($frink == {}) && ($procheck == {})} {
+    if {($frink == {}) && ($procheck == {}) && ($nagelfar == {})
+        && ($tclchecker == {})} {
 	return
     }
     if {$frink    != {}} {
 	run-frink $m
 	puts "------------------------------------------------------"
     }
-    if {$procheck    != {}} {
+    if {$tclchecker != {}} {
+	run-tclchecker $m
+	puts "------------------------------------------------------"
+    } elseif {$procheck != {}} {
 	run-procheck $m
 	puts "------------------------------------------------------"
     }
+    if {$nagelfar    !={}} {
+    	run-nagelfar $m
+	puts "------------------------------------------------------"
+    }
     puts ""
 
     return
@@ -2208,54 +2388,6 @@ proc __approve {} {
 # --------------------------------------------------------------
 # Documentation
 
-proc __html  {} {global argv ; if {![checkmod]} return ; eval gendoc html  html $argv}
-proc __nroff {} {global argv ; if {![checkmod]} return ; eval gendoc nroff n    $argv}
-proc __tmml  {} {global argv ; if {![checkmod]} return ; eval gendoc tmml  tmml $argv}
-proc __text  {} {global argv ; if {![checkmod]} return ; eval gendoc text  txt  $argv}
-proc __wiki  {} {global argv ; if {![checkmod]} return ; eval gendoc wiki  wiki $argv}
-proc __latex {} {global argv ; if {![checkmod]} return ; eval gendoc latex tex  $argv}
-proc __dvi   {} {
-    global argv ; if {![checkmod]} return
-    __latex
-    file mkdir [file join doc dvi]
-    cd         [file join doc dvi]
-    foreach f [glob -nocomplain ../latex/*.tex] {
-	puts "Gen (dvi): $f"
-	exec latex $f 1>@ stdout 2>@ stderr
-    }
-    cd ../..
-}
-proc __ps   {} {
-    global argv ; if {![checkmod]} return
-    __dvi
-    file mkdir [file join doc ps]
-    cd         [file join doc ps]
-    foreach f [glob -nocomplain ../dvi/*.dvi] {
-	puts "Gen (dvi): $f"
-	exec dvips -o [file rootname [file tail $f]].ps $f 1>@ stdout 2>@ stderr
-    }
-    cd ../..
-}
-
-proc __list  {} {
-    global argv ; if {![checkmod]} return
-    eval gendoc list l $argv
-    
-    set FILES [glob -nocomplain doc/list/*.l]
-    set LIST [open [file join doc list manpages.tcl] w]
-
-    foreach file $FILES {
-        set f [open $file r]
-        puts $LIST [read $f]
-        close $f
-    }
-    close $LIST
-
-    eval file delete -force $FILES
-
-    return
-}
-
 proc __desc  {} {
     global argv ; if {![checkmod]} return
     array set pd [getpdesc]
@@ -2269,7 +2401,7 @@ proc __desc  {} {
     puts {Descriptions...}
     if {[llength $argv] == 0} {set argv [modules]}
 
-    foreach m [lsort $argv] {
+    foreach m [lsort [dealias $argv]] {
 	array set _ {}
 	set pkg {}
 	foreach {p vlist} [ppackages $m] {
@@ -2312,7 +2444,7 @@ proc __desc/2  {} {
     puts {Descriptions...}
     if {[llength $argv] == 0} {set argv [modules]}
 
-    foreach m [lsort $argv] {
+    foreach m [lsort [dealias $argv]] {
 	struct::matrix m
 	m add columns 3
 
@@ -2368,7 +2500,7 @@ proc __docstrip/regen {} {
     global argv ; if {![checkmod]} return
     if {[llength $argv] == 0} {set argv [modules]}
 
-    foreach m [lsort $argv] {
+    foreach m [lsort [dealias $argv]] {
 	if {[docstripUser $m]} {
 	    docstripRegen $m
 	}
@@ -2378,20 +2510,45 @@ proc __docstrip/regen {} {
 }
 
 # --------------------------------------------------------------
+## Make sak specific packages visible.
 
-set cmd [lindex $argv 0]
-if {[llength [info procs __$cmd]] == 0} {
-    puts stderr "unknown command $cmd"
-    set fl {}
-    foreach p [lsort [info procs __*]] {
-	lappend fl [string range $p 2 end]
-    }
-    puts stderr "use: [join $fl ", "]"
-    exit 1
-}
+lappend auto_path [file join $distribution support devel sak]
 
+# --------------------------------------------------------------
+## Dispatcher to the sak commands.
+
+set  cmd  [lindex $argv 0]
 set  argv [lrange $argv 1 end]
 incr argc -1
 
+# Prefer a command implementation found in the support tree.
+# Then see if the command is implemented here, in this file.
+# At last fail and report possible commands.
+
+set base  [file dirname [info script]]
+set sbase [file join $base support devel sak]
+set cbase [file join $sbase $cmd]
+set cmdf  [file join $cbase cmd.tcl]
+
+if {[file exists $cmdf] && [file readable $cmdf]} {
+    source $cmdf
+    exit 0
+}
+
+if {[llength [info procs __$cmd]] == 0} {
+    puts stderr "$argv0 : Illegal command \"$cmd\""
+    set fl {}
+    foreach p [info procs __*] {
+	lappend fl [string range $p 2 end]
+    }
+    foreach p [glob -nocomplain -directory $sbase */cmd.tcl] {
+	lappend fl [lindex [file split $p] end-1]
+    }
+
+    regsub -all . $argv0 { } blank
+    puts stderr "$blank : Should have been [linsert [join [lsort -uniq $fl] ", "] end-1 or]"
+    exit 1
+}
+
 __$cmd
 exit 0
