@@ -1,4 +1,4 @@
-package_version 1.9
+package_version 1.10
 package_name    tcllib
 
 dist_exclude    config
@@ -15,8 +15,11 @@ critcl      md5c         md5/md5c.tcl
 critcl      md5cryptc    md5crypt/md5cryptc.tcl
 critcl      rc4c         rc4/rc4c.tcl
 critcl      sha1c        sha1/sha1c.tcl
+critcl      sha256c      sha1/sha256c.tcl
 critcl      uuid         uuid/uuid.tcl
 critcl      struct_treec struct/tree_c.tcl
+critcl      struct_graphc struct/graph_c.tcl
+critcl      struct_setc  struct/sets_c.tcl
 critcl      base32c      base32/base32_c.tcl
 critcl      base32hexc   base32/base32hex_c.tcl
 critcl      ipMorec      dns/ipMoreC.tcl
